// Deelx.cpp : Implementation of CDeelx
//
// Copyright 2006 (c) RegExLab.com
//
// Author: ʷΰ (sswater shi)
//
// 2006/06/20 03:17:12
//

#include "stdafx.h"
#include "RegDeelx.h"
#include "Deelx.h"

/////////////////////////////////////////////////////////////////////////////
// CDeelx

STDMETHODIMP CDeelx::InterfaceSupportsErrorInfo(REFIID riid)
{
	static const IID* arr[] = 
	{
		&IID_IDeelx,
	};
	for (int i=0;i<sizeof(arr)/sizeof(arr[0]);i++)
	{
		if (InlineIsEqualGUID(*arr[i],riid))
			return S_OK;
	}
	return S_FALSE;
}

STDMETHODIMP CDeelx::Compile(BSTR pattern, BSTR flags)
{
	int flagslen = wcslen(flags);
	m_nflags = 0;

	for(int i=0; i<flagslen; i++)
	{
		if     (flags[i] == L'i' || flags[i] == L'I') m_nflags |= IGNORECASE ;
		else if(flags[i] == L's' || flags[i] == L'S') m_nflags |= SINGLELINE ;
		else if(flags[i] == L'g' || flags[i] == L'G') m_nflags |= GLOBAL     ;
		else if(flags[i] == L'm' || flags[i] == L'M') m_nflags |= MULTILINE  ;
		else if(flags[i] == L'r' || flags[i] == L'R') m_nflags |= RIGHTTOLEFT;
		else if(flags[i] == L'x' || flags[i] == L'X') m_nflags |= EXTENDED   ;
	}

	m_regexp.Compile(pattern, m_nflags);

	return S_OK;
}

STDMETHODIMP CDeelx::Match(BSTR text, int start, VARIANT_BOOL * pVal)
{
	int length = wcslen(text);

	// check
	if(m_pscopy == NULL || wcscmp(m_pscopy, text))
	{
		// release
		if(m_pscopy != NULL)
		{
			delete [] m_pscopy;
			m_pscopy = NULL;
		}

		// copy
		m_pscopy = new wchar_t[length + 1];
		wcscpy(m_pscopy, text);
		m_pscopy[length] = 0;

		// reset pos
		m_nstart = -1;
	}

	// begin pos
	if(start < 0)
	{
		if((m_nflags & GLOBAL) && (m_nstart >= 0))
			start = m_nstart;
	}

	// match
	m_result = m_regexp.Match(text, length, start);

	// release
	if(m_result.IsMatched())
		m_nstart = (m_nflags & RIGHTTOLEFT) ? m_result.GetStart() : m_result.GetEnd();
	else
	{
		m_nstart = -1;
	}

	*pVal = m_result.IsMatched() ? -1 : 0;

	return S_OK;
}

STDMETHODIMP CDeelx::Replace(BSTR text, BSTR replaceto, int start, int times, BSTR * pVal)
{
	// result
	wchar_t * pResult = m_regexp.Replace(text, replaceto, start, times);

	*pVal = SysAllocString(pResult);

	delete [] pResult;

	return S_OK;
}

STDMETHODIMP CDeelx::get_NextStart(long *pVal)
{
	*pVal = m_nstart;

	return S_OK;
}

/*
STDMETHODIMP CDeelx::put_NextStart(long newVal)
{
	m_nstart = newVal;

	return S_OK;
}
*/

STDMETHODIMP CDeelx::get_IsMatched(VARIANT_BOOL *pVal)
{
	// result
	*pVal = m_result.IsMatched() ? -1 : 0;

	return S_OK;
}

STDMETHODIMP CDeelx::get_Begin(long *pVal)
{
	*pVal = m_result.GetStart();

	return S_OK;
}

STDMETHODIMP CDeelx::get_End(long *pVal)
{
	*pVal = m_result.GetEnd();

	return S_OK;
}

STDMETHODIMP CDeelx::get_Group(int number, BSTR *pVal)
{
	int start = m_result.GetGroupStart(number);

	if(start >= 0)
	{
		wchar_t * pstr = m_pscopy + start;
		int length = m_result.GetGroupEnd(number) - start;

		wchar_t tempc = pstr[length];
		pstr[length] = 0;

		*pVal = SysAllocString(pstr);

		pstr[length] = tempc;
	}
	else
	{
		*pVal = SysAllocString(L"");
	}

	return S_OK;
}

STDMETHODIMP CDeelx::get_GroupBegin(int number, long *pVal)
{
	*pVal = m_result.GetGroupStart(number);

	return S_OK;
}

STDMETHODIMP CDeelx::get_GroupEnd(int number, long *pVal)
{
	*pVal = m_result.GetGroupEnd(number);

	return S_OK;
}

STDMETHODIMP CDeelx::get_MaxGroupNumber(long *pVal)
{
	*pVal = m_result.MaxGroupNumber();

	return S_OK;
}

STDMETHODIMP CDeelx::get_MatchResult(BSTR *pVal)
{
	get_Group(0, pVal);

	return S_OK;
}
