// BackrefFromElxT.h
//
// Copyright 2006 (c) RegExLab.com
//
// Author: ʷΰ (sswater shi)
//
// 2006/06/20 03:17:12
//

#ifndef _BACKREF_FROM_ELX_H__
#define _BACKREF_FROM_ELX_H__

#include "ElxInterface.h"
#include "StringRef.h"

template <typename CHART> class CBackrefFromElxT : public ElxInterface
{
public:
	int Match(ContextInterface * pContext)
	{
		unsigned int nStart, nEnd;
		int captured = pContext->GetBackref(m_nbackref, nStart, nEnd);

		// check captured
		if(!captured)
			return 0;

		unsigned int nMin = nStart < nEnd ? nStart : nEnd;
		unsigned int nMax = nStart > nEnd ? nStart : nEnd;

		const CHART * pcsz   = (const CHART *)pContext->GetTarget();
		int           nBegin = pContext->GetCurrentPos();

		int bsucc   = 0;
		int nLength = nMax - nMin;

		if(m_bRightLeft)
		{
			// check pos
			if(nBegin < nLength)
				return 0;

			// compare
			StringRefT <CHART> str(pcsz + nMin, nLength);

			if(m_bIgnoreCase)
				bsucc = ! str.nCompareNoCase(pcsz + (nBegin - nLength));
			else
				bsucc = ! str.nCompare(pcsz + (nBegin - nLength));

			if(bsucc)
				pContext->SetCurrentPos(pContext->GetCurrentPos() - nLength);
		}
		else
		{
			// check pos
			if(nBegin + nLength > pContext->GetLength())
				return 0;

			// compare string
			StringRefT <CHART> str(pcsz + nMin, nLength);

			if(m_bIgnoreCase)
				bsucc = ! str.nCompareNoCase(pcsz + nBegin);
			else
				bsucc = ! str.nCompare(pcsz + nBegin);

			if(bsucc)
				pContext->SetCurrentPos(pContext->GetCurrentPos() + nLength);
		}

		return bsucc;
	}

	int MatchNext(ContextInterface * pContext)
	{
		unsigned int nStart, nEnd;
		int captured = pContext->GetBackref(m_nbackref, nStart, nEnd);

		if(captured)
		{
			unsigned int nMin = nStart < nEnd ? nStart : nEnd;
			unsigned int nMax = nStart > nEnd ? nStart : nEnd;

			int nLength = nMax - nMin;

			pContext->SetCurrentPos(pContext->GetCurrentPos() + (m_bRightLeft ? nLength : -nLength));
		}

		return 0;
	}

public:
	CBackrefFromElxT(int nbackref, int bRightLeft, int bIgnoreCase)
	{
		m_nbackref   = nbackref;
		m_bRightLeft = bRightLeft;
	}

protected:
	int m_nbackref;
	int m_bRightLeft;
	int m_bIgnoreCase;
};

#endif//_BACKREF_FROM_ELX_H__
