// BoundaryElx.h
//
// Copyright 2006 (c) RegExLab.com
//
// Author: ʷΰ (sswater shi)
//
// 2006/07/05 08:45:51
//

#ifndef _BOUNDARY_ELX_H__
#define _BOUNDARY_ELX_H__

#include "ElxInterface.h"
#include "StringRef.h"

enum BOUNDARY_TYPE {
	FILE_BEGIN, // begin of whole text
	FILE_END  , // end of whole text
	LINE_BEGIN, // begin of line
	LINE_END  , // end of line
	WORD_BEGIN, // begin of word
	WORD_END  , // end of word
	WORD_EDGE ,
};

template <typename CHART> class CBoundaryElxT : public ElxInterface
{
public:
	int Match(ContextInterface * pContext)
	{
		// info
		unsigned int nBegin = pContext->GetCurrentPos();
		const CHART * target = (const CHART *)pContext->GetTarget();
		int nLength = pContext->GetLength();

		// char
		CHART chL = nBegin > 0       ? target[nBegin-1] : 0;
		CHART chR = nBegin < nLength ? target[nBegin  ] : 0;

		// run
		int bsucc;

		switch(m_nType)
		{
		case FILE_BEGIN:
			bsucc = (nBegin == 0);
			break;

		case FILE_END:
			bsucc = (nBegin == nLength);
			break;

		case LINE_BEGIN:
			bsucc = (nBegin == 0) || (chL == RCHART('\n'));
			break;

		case LINE_END:
			bsucc = (nBegin == nLength) || (chR == RCHART('\n'));
			break;

		case WORD_BEGIN:
			bsucc = !IsWord(chL) && IsWord(chR);
			break;

		case WORD_END:
			bsucc = IsWord(chL) && !IsWord(chR);
			break;

		case WORD_EDGE:
			bsucc = IsWord(chL) ? !IsWord(chR) : IsWord(chR);
			break;
		}

		return m_bYes ? bsucc : !bsucc;
	}

	int MatchNext(ContextInterface * pContext) { return 0; }

public:
	CBoundaryElxT(BOUNDARY_TYPE nType, int bYes = 1)
	{
		m_nType = nType;
		m_bYes  = bYes;
	}

protected:
	inline int IsWord(CHART ch) { return (ch >= RCHART('A') && ch <= RCHART('Z')) || (ch >= RCHART('a') && ch <= RCHART('z')) || (ch >= RCHART('0') && ch <= RCHART('9')) || (ch == RCHART('_')); }

protected:
	BOUNDARY_TYPE m_nType;
	int m_bYes;
};

#endif//_BOUNDARY_ELX_H__
