// ContextInterface.h
//
// Copyright 2006 (c) RegExLab.com
//
// Author: ʷΰ (sswater shi)
//
// 2006/06/20 03:17:12
//

#ifndef _CONTEXT_INTERFACE_H__
#define _CONTEXT_INTERFACE_H__

#include "StackInterface.h"

#ifndef _REGEX_FLAGS_DEFINED
	enum REGEX_FLAGS
	{
		NO_FLAG        = 0,
		SINGLELINE     = 0x01,
		MULTILINE      = 0x02,
		GLOBAL         = 0x04,
		IGNORECASE     = 0x08,
		RIGHTTOLEFT    = 0x10,
		EXTENDED       = 0x20,
	};
	#define _REGEX_FLAGS_DEFINED
#endif

// state
struct ContextState
{
	StackState stackstate;
	unsigned int m_nCurrentPos;
};

class ContextInterface
{
// pos
public:
	virtual void SetCurrentPos(unsigned int nPos) = 0;
	virtual unsigned int GetCurrentPos() = 0;

	// position for '\G'
	virtual void SetGlobalPos(unsigned int nPos) = 0;
	virtual unsigned int GetGlobalPos() = 0;

	virtual void SetLastLength(int nLength) = 0;
	virtual int GetLastLength() = 0;

// stack
public:
	virtual void Push(unsigned int   n, unsigned int id) = 0;
	virtual int  Pop (unsigned int & n, unsigned int id) = 0;

// state
public:
	virtual void Mark   (ContextState & state) = 0;
	virtual void Restore(ContextState   state, int stackonly = 0) = 0;

// backref
public:
	virtual void PushBackrefStart(int nbackref, unsigned int nStart, int bNoPush = 0) = 0;
	virtual void PushBackrefEnd  (int nbackref, unsigned int nEnd) = 0;

	virtual void PopBackrefStart (int nbackref) = 0;
	virtual void PopBackrefEnd   (int nbackref) = 0;

	virtual int  GetBackref(int nbackref, unsigned int & nStart, unsigned int & nEnd) = 0;

	virtual int  GetLastParenMatch() = 0;

// initialize
public:
	virtual void   ResetAll(void * pTarget, int nLength) = 0;
	virtual void   ResetStack() = 0;

	virtual void * GetTarget() = 0;
	virtual int    GetLength() = 0;

// destructor
public:
	virtual ~ContextInterface(){};
};

#endif//_CONTEXT_INTERFACE_H__
