// DeelxRegexp.h: interface for the CDeelxRegexp class.
//
// Copyright 2006 (c) RegExLab.com
//
// Author: ʷΰ (sswater shi)
//
// 2006/06/20 03:17:12
//

#ifndef AFX_DEELXREGEXP_H__
#define AFX_DEELXREGEXP_H__

class ContextInterface;

#ifndef _REGEX_FLAGS_DEFINED
	enum REGEX_FLAGS
	{
		NO_FLAG        = 0,
		SINGLELINE     = 0x01,
		MULTILINE      = 0x02,
		GLOBAL         = 0x04,
		IGNORECASE     = 0x08,
		RIGHTTOLEFT    = 0x10,
		EXTENDED       = 0x20,
	};
	#define _REGEX_FLAGS_DEFINED
#endif

// wchar_t
#if !defined(_WCHAR_T_DEFINED) && defined(_MSC_VER)
	typedef unsigned short wchar_t;
	#define _WCHAR_T_DEFINED
#endif

//
// match result
//
class MatchResult
{
public:
	int IsMatched();

public:
	int GetStart();
	int GetEnd  ();

public:
	int MaxGroupNumber();
	int GetGroupStart(int nGroupNumber);
	int GetGroupEnd  (int nGroupNumber);

public:
	MatchResult(void *, int = 0);
	virtual ~MatchResult();

	MatchResult & operator = (const MatchResult &);
	inline operator int() { return IsMatched(); }

private:
	void * m_pPreserved;
};

//
// regexp template
//
template <typename CHART> class CDeelxRegexpT
{
public:
	CDeelxRegexpT(const CHART * pattern = 0, int flags = 0);
	CDeelxRegexpT(const CHART * pattern, int length, int flags);

public:
	void Compile(const CHART * pattern, int flags = 0);
	void Compile(const CHART * pattern, int length, int flags);

public:
	MatchResult MatchExact(const CHART * tstring);
	MatchResult MatchExact(const CHART * tstring, int length);

public:
	MatchResult Match(const CHART * tstring, int start = -1);
	MatchResult Match(const CHART * tstring, int length, int start);

public:
	ContextInterface * PrepareMatch(const CHART * tstring, int start = -1);
	ContextInterface * PrepareMatch(const CHART * tstring, int length, int start);

	MatchResult Match(ContextInterface * pContext);

	void ReleaseContext(ContextInterface * pContext);

public:
	CHART * Replace(const CHART * tstring, const CHART * replaceto, int start = -1, int ntimes = -1);
	static CHART * Replace(const CHART * tstring, const CHART * pattern, const CHART * replaceto, int flags = 0, int start = -1, int ntimes = -1);

public:
	~CDeelxRegexpT();

protected:
	static int tstringlen(const CHART * tstring);

protected:
	void * m_pPreserved;
};

//
// regexp
//
typedef CDeelxRegexpT <wchar_t> CDeelxRegexpW;
typedef CDeelxRegexpT <char>    CDeelxRegexpA;

#ifdef _UNICODE
	typedef CDeelxRegexpW CDeelxRegexp;
#else
	typedef CDeelxRegexpA CDeelxRegexp;
#endif


#endif//AFX_DEELXREGEXP_H__
