// DeelxRegexpT.inl
//
// Copyright 2006 (c) RegExLab.com
//
// Author: ʷΰ (sswater shi)
//
// 2006/06/20 03:17:12
//

#include "DeelxRegexp.h"
#include "PreBuilder.h"

////

template <typename CHART> CDeelxRegexpT <CHART>::CDeelxRegexpT(const CHART * pattern, int flags)
{
	m_pPreserved = 0;
	Compile(pattern, flags);
}

template <typename CHART> CDeelxRegexpT <CHART>::CDeelxRegexpT(const CHART * pattern, int length, int flags)
{
	m_pPreserved = 0;
	Compile(pattern, length, flags);
}

////

template <typename CHART> void CDeelxRegexpT <CHART>::Compile(const CHART * pattern, int length, int flags)
{
	if(pattern != 0)
	{
		CPreBuilderT <CHART> * pBuilder = 0;
		
		if(m_pPreserved != 0)
			pBuilder = (CPreBuilderT <CHART> *) m_pPreserved;
		else
			m_pPreserved = pBuilder = new CPreBuilderT <CHART>();

		pBuilder->Clear();
		pBuilder->Build(StringRefT <CHART> (pattern, length), flags);
	}
}

template <typename CHART> MatchResult CDeelxRegexpT <CHART>::MatchExact(const CHART * tstring, int length)
{
	if(m_pPreserved != 0)
	{
		// elx
		CPreBuilderT <CHART> * pBuilder = (CPreBuilderT <CHART> *)m_pPreserved;
		ElxInterface * pElx = pBuilder->GetTopElx();

		// context
		CRegexContext context;
		context.ResetAll((void*)tstring, length);

		// pos
		unsigned int startpos = 0, endpos = 0;

		if(pBuilder->GetFlags() & RIGHTTOLEFT)
		{
			startpos = length;
			endpos   = 0;
		}
		else
		{
			startpos = 0;
			endpos   = length;
		}

		// current pos & global point
		context.SetCurrentPos(startpos);
		context.SetGlobalPos(context.GetCurrentPos());

		// for '\0'
		context.PushBackrefStart(0, context.GetCurrentPos());

		// match
		if(!pElx->Match(&context))
		{
			return 0;
		}
		else
		{
			int blastzero = 0;

			while(context.GetCurrentPos() != endpos)
			{
				if(!pElx->MatchNext(&context))
				{
					return 0;
				}
				else
				{
					if(context.GetCurrentPos() == startpos)
					{
						if(blastzero) // second time zero width
							return 0;

						blastzero = 1;
					}
					else
					{
						blastzero = 0;
					}
				}
			}
		}

		// construct matchresult
		return MatchResult(&context, pBuilder->GetMaxBackref());
	}
	else
	{
		return 0;
	}
}

template <typename CHART> MatchResult CDeelxRegexpT <CHART>::Match(const CHART * tstring, int length, int start)
{
	if(m_pPreserved != 0)
	{
		// elx
		CPreBuilderT <CHART> * pBuilder = (CPreBuilderT <CHART> *)m_pPreserved;
		ElxInterface * pElx = pBuilder->GetTopElx();

		// context
		CRegexContext context;
		context.ResetAll((void*)tstring, length);

		if(start < 0)
		{
			if(pBuilder->GetFlags() & RIGHTTOLEFT)
				context.SetCurrentPos(length);
			else
				context.SetCurrentPos(0);
		}
		else
		{
			context.SetCurrentPos(start);
		}

		// construct matchresult
		return Match(&context);
	}
	else
	{
		return 0;
	}
}

template <typename CHART> ContextInterface * CDeelxRegexpT <CHART>::PrepareMatch(const CHART * tstring, int length, int start)
{
	if(m_pPreserved != 0)
	{
		// builder
		CPreBuilderT <CHART> * pBuilder = (CPreBuilderT <CHART> *)m_pPreserved;

		// context
		CRegexContext * pContext = new CRegexContext();
		pContext->ResetAll((void*)tstring, length);

		if(start < 0)
		{
			if(pBuilder->GetFlags() & RIGHTTOLEFT)
				pContext->SetCurrentPos(length);
			else
				pContext->SetCurrentPos(0);
		}
		else
		{
			pContext->SetCurrentPos(start);
		}

		// context
		return pContext;
	}
	else
	{
		return 0;
	}
}

template <typename CHART> void CDeelxRegexpT <CHART>::ReleaseContext(ContextInterface * pContext)
{
	if(pContext != 0)
	{
		delete pContext;
	}
}

template <typename CHART> MatchResult CDeelxRegexpT <CHART>::Match(ContextInterface * pContext)
{
	if(m_pPreserved != 0 && pContext != 0)
	{
		// elx
		CPreBuilderT <CHART> * pBuilder = (CPreBuilderT <CHART> *)m_pPreserved;
		ElxInterface * pElx = pBuilder->GetTopElx();

		// check & correct pos
		if(pContext->GetCurrentPos() > (unsigned int)pContext->GetLength())
			pContext->SetCurrentPos(pContext->GetLength());

		// global point
		pContext->SetGlobalPos(pContext->GetCurrentPos());

		int bmatched = 0;
		unsigned int start;
		int brtol = pBuilder->GetFlags() & RIGHTTOLEFT;

		while(brtol ? (pContext->GetCurrentPos() > 0) : (pContext->GetCurrentPos() < (unsigned int)pContext->GetLength()))
		{
			// check & correct pos
			if(pContext->GetCurrentPos() > (unsigned int)pContext->GetLength())
				pContext->SetCurrentPos(pContext->GetLength());

			// each try
			start = pContext->GetCurrentPos();

			pContext->ResetStack();
			pContext->PushBackrefStart(0, start);

			// match
			bmatched = pElx->Match(pContext);

			// ok or next
			if(bmatched)
			{
				int nLength     = pContext->GetCurrentPos() - start;
				int nLastLength = pContext->GetLastLength();

				pContext->SetLastLength(nLength);

				if(nLength != 0 || nLastLength != 0)
					break;
			}

			pContext->SetCurrentPos(start + (brtol ? -1 : 1));
		}

		if(bmatched)
			return MatchResult(pContext, pBuilder->GetMaxBackref());
		else
			return 0;
	}
	else
	{
		return 0;
	}
}

template <typename CHART> CDeelxRegexpT <CHART>::~CDeelxRegexpT()
{
	if(m_pPreserved != 0)
	{
		CPreBuilderT <CHART> * pBuilder = (CPreBuilderT <CHART> *) m_pPreserved;
		delete pBuilder;
		m_pPreserved = 0;
	}
}

//////

template <typename CHART> void CDeelxRegexpT <CHART>::Compile(const CHART * pattern, int flags)
{
	Compile(pattern, tstringlen(pattern), flags);
}

template <typename CHART> MatchResult CDeelxRegexpT <CHART>::MatchExact(const CHART * tstring)
{
	return MatchExact(tstring, tstringlen(tstring));
}

template <typename CHART> MatchResult CDeelxRegexpT <CHART>::Match(const CHART * tstring, int start)
{
	return Match(tstring, tstringlen(tstring), start);
}

template <typename CHART> ContextInterface * CDeelxRegexpT <CHART>::PrepareMatch(const CHART * tstring, int start)
{
	return PrepareMatch(tstring, tstringlen(tstring), start);
}

////

template <typename CHART> int CDeelxRegexpT <CHART>::tstringlen(const CHART * tstring)
{
	if(tstring == 0)
		return 0;

	int count = 0;
	while(tstring[count] != ((CHART)'\0')) count ++;

	return count;
}

////

template <typename CHART> CHART * CDeelxRegexpT <CHART>::Replace(const CHART * tstring, const CHART * replaceto, int start, int ntimes)
{
	typedef StringRefT <CHART> StringRef;

	if(m_pPreserved != 0)
	{
		// prepare
		CPreBuilderT <CHART> * pBuilder = (CPreBuilderT <CHART> *)m_pPreserved;
		ContextInterface * pContext = PrepareMatch(tstring, start);

		// info
		int nMaxBackref = pBuilder->GetMaxBackref();
		int flags       = pBuilder->GetFlags();
		int lastIndex   = (flags & RIGHTTOLEFT) ? pContext->GetLength() : 0;
		int endpos      = (flags & RIGHTTOLEFT) ? 0 : pContext->GetLength();

		std::list <StringRef> buffer;

		// match
		for(int ntime = 0; ntimes < 0 || ntime < ntimes; ntime ++)
		{
			MatchResult result = Match(pContext);

			if(!result.IsMatched())
				break;

			// before matched part
			if((flags & RIGHTTOLEFT) != 0)
			{
				if(result.GetEnd() < lastIndex)
				{
					// insert head
					buffer.insert(buffer.begin(),
						StringRef(tstring + result.GetEnd(), lastIndex - result.GetEnd())
					);
				}

				lastIndex = result.GetStart();
			}
			else
			{
				if(lastIndex < result.GetStart())
				{
					// append tail
					buffer.insert(buffer.end(),
						StringRef(tstring + lastIndex, result.GetStart() - lastIndex)
					);
				}

				lastIndex = result.GetEnd();
			}

			// matched
			static const CHART rtoptn[] = {
				RCHART('\\'), RCHART('$' ), RCHART('('), RCHART('?'), RCHART(':'), RCHART('[' ),
					RCHART('$' ),
					RCHART('&' ),
					RCHART('`' ),
					RCHART('\''), RCHART('+'),
					RCHART('_' ),
					RCHART('\\'), RCHART('d'),
				RCHART(']'), RCHART(')' ), RCHART('\0')
			};
			static CDeelxRegexpT <CHART> rtoreg(rtoptn);

			ContextInterface * pContext2 = rtoreg.PrepareMatch(replaceto);
			int lastIndex2 = 0;
			int endpos2 = tstringlen(replaceto);
			typename std::list <StringRef> ::iterator inspnt = (flags & RIGHTTOLEFT) ? buffer.begin() : buffer.end();

			while(2)
			{
				MatchResult result2 = rtoreg.Match(pContext2);

				if(!result2.IsMatched())
					break;

				if(lastIndex2 < result2.GetStart())
				{
					buffer.insert(inspnt,
						StringRef(replaceto + lastIndex2, result2.GetStart() - lastIndex2)
					);

					lastIndex2 = result2.GetStart();
				}

				int delta = 2, nmatch = 0;

				// special
				switch(replaceto[result2.GetStart() + 1])
				{
				case RCHART('$'):
					buffer.insert(inspnt,
						StringRef(rtoptn + 1, 1) // '$' itself
					);
					break;

				case RCHART('&'):
					buffer.insert(inspnt,
						StringRef(tstring + result.GetStart(), result.GetEnd() - result.GetStart())
					);
					break;

				case RCHART('`'):
					buffer.insert(inspnt,
						StringRef(tstring, result.GetStart())
					);
					break;

				case RCHART('\''):
					buffer.insert(inspnt,
						StringRef(tstring + result.GetEnd(), pContext->GetLength() - result.GetEnd())
					);
					break;

				case RCHART('+'):
					nmatch = pContext->GetLastParenMatch();
					if(nmatch <= result.MaxGroupNumber())
					{
						buffer.insert(inspnt,
							StringRef(tstring + result.GetGroupStart(nmatch), result.GetGroupEnd(nmatch) - result.GetGroupStart(nmatch))
						);
					}
					break;

				case RCHART('_'):
					buffer.insert(inspnt,
						StringRef(tstring, pContext->GetLength())
					);
					break;

				default:
					nmatch = 0;
					for(delta=1; delta<=3; delta++)
					{
						CHART ch = replaceto[lastIndex2 + delta];

						if(ch < RCHART('0') || ch > RCHART('9'))
							break;

						nmatch = nmatch * 10 + (ch - RCHART('0'));
					}

					if(nmatch > nMaxBackref)
					{
						while(nmatch > nMaxBackref)
						{
							nmatch /= 10;
							delta --;
						}

						if(nmatch == 0)
						{
							delta = 1;
						}
					}

					if(delta == 1)
					{
						buffer.insert(inspnt,
							StringRef(rtoptn + 1, 1) // '$' itself
						);
					}
					else
					{
						buffer.insert(inspnt,
							StringRef(tstring + result.GetGroupStart(nmatch), result.GetGroupEnd(nmatch) - result.GetGroupStart(nmatch))
						);
					}
					break;
				}

				lastIndex2 += delta;
			}

			if(lastIndex2 < endpos2)
			{
				buffer.insert(inspnt,
					StringRef(replaceto + lastIndex2, endpos2 - lastIndex2)
				);
			}

			rtoreg.ReleaseContext(pContext2);
		}

		if((flags & RIGHTTOLEFT) != 0)
		{
			if(endpos < lastIndex)
			{
				buffer.insert(buffer.begin(),
					StringRef(tstring + endpos, lastIndex - endpos)
				);
			}
		}
		else
		{
			if(lastIndex < endpos)
			{
				buffer.insert(buffer.end(),
					StringRef(tstring + lastIndex, endpos - lastIndex)
				);
			}
		}

		ReleaseContext(pContext);

		// join string
		int strlength = 0;
		typename std::list <StringRef> ::iterator it;

		for(it=buffer.begin(); it!=buffer.end(); it++)
		{
			StringRef & strref = *it;
			strlength += strref.GetLength();
		}

		// alloc
		CHART * pString = new CHART[strlength + 1];
		strlength = 0;

		for(it=buffer.begin(); it!=buffer.end(); it++)
		{
			StringRef & strref = *it;
			memcpy(pString + strlength, (const CHART *)strref, strref.GetLength() * sizeof(CHART));
			strlength += strref.GetLength();
		}

		pString[strlength] = RCHART('\0');

		return pString;
	}

	return 0;
}

template <typename CHART> CHART * CDeelxRegexpT <CHART>::Replace(const CHART * tstring, const CHART * pattern, const CHART * replaceto, int flags, int start, int ntimes)
{
	return CDeelxRegexpT <CHART> (pattern, flags).Replace(tstring, replaceto, start, ntimes);
}
