// PackedStack.cpp : packed stack
//
// Copyright 2006 (c) RegExLab.com
//
// Author: ʷΰ (sswater shi)
//
// 2006/06/20 03:17:12
//

#include <memory.h>
#include "PackedStack.h"

CPackedStack::CPackedStack(int nInitialSize)
{
	m_nBufSize   = nInitialSize;
	m_pBuffer    = m_nBufSize ? new unsigned char[m_nBufSize] : 0;
	m_nSize      = 0;
	m_nCurrentId = 0;
#ifdef _DEBUG
	memset(m_pBuffer, 0, m_nBufSize);
#endif
}

CPackedStack::~CPackedStack()
{
	delete [] m_pBuffer;
}

void CPackedStack::Push(unsigned int   n, unsigned int id)
{
	int need = BytesNeed(n);

	// check id
	if(m_nCurrentId != id)
	{
		if(m_nSize > 0)
		{
			// enlarge
			if(m_nSize + need + 2 + 4 > m_nBufSize)
				EnLarge();

			*(short*)(m_pBuffer+m_nSize) = ((short)m_nCurrentId & 0x3FFF) | 0x8000;
			m_nSize += 2;
		}
		m_nCurrentId = id;
	}
	else if(m_nSize > 0)
	{
		// repeat
		unsigned int lastn;

		if(IsRepeat())
		{
			lastn = ReadValue(m_nSize-1, BytesUsed(m_nSize-1));
			if(lastn == n)
			{
				if(GetRepeat() < REPEAT_MAX)
				{
					IncRepeat();
					return;
				}
			}
		}
		else
		{
			lastn = ReadValue(m_nSize, BytesUsed(m_nSize));
			if(lastn == n)
			{
				// enlarge
				if(m_nSize + 1 + 4 > m_nBufSize)
					EnLarge();

				m_pBuffer[m_nSize++] = 0x41;
				return;
			}
		}
	}

	// enlarge
	if(m_nSize + need + 4 > m_nBufSize)
		EnLarge();

	// save
	WriteValue(n, need);
	m_nSize += need;
}

int  CPackedStack::Pop (unsigned int & n, unsigned int id)
{
	if(m_nCurrentId != id || m_nSize == 0)
		return 0;

	if(IsRepeat())
	{
		n = ReadValue(m_nSize-1, BytesUsed(m_nSize-1));

		DecRepeat();
		if(GetRepeat() == 0)
			m_nSize --;
	}
	else
	{
		int used = BytesUsed(m_nSize);
		n = ReadValue(m_nSize, used);
		m_nSize -= used;

		if(m_nSize > 0 && IsIddata())
		{
			m_nCurrentId = (*(short*)(m_pBuffer+m_nSize-2)) & 0x3FFF;
			m_nSize -= 2;
		}
	}

	return 1;
}

int  CPackedStack::Peek(unsigned int & n, unsigned int id)
{
	if(m_nCurrentId != id || m_nSize == 0)
		return 0;

	if(IsRepeat())
		n = ReadValue(m_nSize-1, BytesUsed(m_nSize-1));
	else
		n = ReadValue(m_nSize, BytesUsed(m_nSize));

	return 1;
}

void CPackedStack::EnLarge()
{
	// new size
	int nNewSize = m_nBufSize * 2;
	if( nNewSize < 8 )
		nNewSize = 8;

	// alloc
	unsigned char * pNewBuf = new unsigned char[nNewSize];
#ifdef _DEBUG
	memset(pNewBuf, 0, nNewSize);
#endif

	// delete old
	if(m_pBuffer != 0)
	{
		memcpy(pNewBuf, m_pBuffer, m_nBufSize);
		delete [] m_pBuffer;
	}

	m_nBufSize = nNewSize;
	m_pBuffer  = pNewBuf;
}

int CPackedStack::BytesNeed(unsigned int n)
{
	if(n <= 0x3F) // 6 bits
		return 1;
	else if(n <= 0x1FFF) // 13 bits
		return 2;
	else if(n <= 0x0FFFFF) // 20 bits
		return 3;
	else if(n <= 0x07FFFFFF) // 27 bits
		return 4;
	else
		return 5;
}

int CPackedStack::BytesUsed(int nSize)
{
	unsigned char & c = m_pBuffer[nSize-1];

	if((c & 0x80) == 0)
		return 1;
	else if((c & 0xE0) == 0xC0)
		return 2;
	else if((c & 0xF0) == 0xE0)
		return 3;
	else if((c & 0xF8) == 0xF0)
		return 4;
	else
		return 5;
}

void CPackedStack::WriteValue(unsigned int n, int need)
{
	unsigned char * bytes = m_pBuffer + m_nSize;
	unsigned int  & data  = *(unsigned int *)bytes;

	data = n;

	switch(need)
	{
		case 2: bytes[1] |= 0xC0; break;
		case 3: bytes[2] |= 0xE0; break;
		case 4: bytes[3] |= 0xF0; break;
		case 5: bytes[4]  = 0xF8; break;
	}
}

unsigned int CPackedStack::ReadValue(int nSize, int used)
{
	unsigned char * bytes = m_pBuffer + nSize - used;
	unsigned int    data  = *(unsigned int *)bytes;

	switch( used )
	{
		case 1: data &= 0x0000003F; break;
		case 2: data &= 0x00001FFF; break;
		case 3: data &= 0x000FFFFF; break;
		case 4: data &= 0x07FFFFFF; break;
	}

	return data;
}

void CPackedStack::Mark(StackState & state)
{
	state.nSize = m_nSize;
	state.nId   = m_nCurrentId;
}

void CPackedStack::Restore(StackState state)
{
	m_nSize      = state.nSize;
	m_nCurrentId = state.nId;
}
