// PosixElx.h
//
// Copyright 2006 (c) RegExLab.com
//
// Author: ʷΰ (sswater shi)
//
// 2006/06/20 03:17:12
//

#ifndef _POSIX_ELX_T_H__
#define _POSIX_ELX_T_H__

#include "ElxInterface.h"
#include "ctype.h"

template <typename CHART> class CPosixElxT : public ElxInterface
{
public:
	int Match(ContextInterface * pContext)
	{
		const CHART * pcsz   = (const CHART *)pContext->GetTarget();
		unsigned int  nBegin = pContext->GetCurrentPos();

		// check pos
		if( m_bRightLeft ? (nBegin == 0) : (nBegin >= pContext->GetLength()) )
			return 0;

		const CHART & pc = pcsz[m_bRightLeft ? nBegin-1 : nBegin];

		int bsucc = m_posixfun != 0 ? (*m_posixfun)(pc) : 0;
		bsucc     = m_bYes ? bsucc : !bsucc;

		if(bsucc) pContext->SetCurrentPos(nBegin + (m_bRightLeft ? -1 : 1) );

		return bsucc;
	}

	int MatchNext(ContextInterface * pContext)
	{
		pContext->SetCurrentPos(pContext->GetCurrentPos() - (m_bRightLeft ? -1 : 1));
		return 0;
	}

public:
	CPosixElxT(const char * posix, int bRightLeft = 0)
	{
		m_bRightLeft = bRightLeft;

		if(posix[1] == '^')
		{
			m_bYes = 0;
			posix += 2;
		}
		else
		{
			m_bYes = 1;
			posix += 1;
		}

		if     (!strncmp(posix, "alnum:", 6)) m_posixfun = isalnum ;
		else if(!strncmp(posix, "alpha:", 6)) m_posixfun = isalpha ;
		else if(!strncmp(posix, "ascii:", 6)) m_posixfun = isascii ;
		else if(!strncmp(posix, "cntrl:", 6)) m_posixfun = iscntrl ;
		else if(!strncmp(posix, "digit:", 6)) m_posixfun = isdigit ;
		else if(!strncmp(posix, "graph:", 6)) m_posixfun = isgraph ;
		else if(!strncmp(posix, "lower:", 6)) m_posixfun = islower ;
		else if(!strncmp(posix, "print:", 6)) m_posixfun = isprint ;
		else if(!strncmp(posix, "punct:", 6)) m_posixfun = ispunct ;
		else if(!strncmp(posix, "space:", 6)) m_posixfun = isspace ;
		else if(!strncmp(posix, "upper:", 6)) m_posixfun = isupper ;
		else if(!strncmp(posix, "xdigit:",7)) m_posixfun = isxdigit;
		else if(!strncmp(posix, "blank:", 6)) m_posixfun = m_isblank;
		else                                   m_posixfun = 0       ;
	}

protected:
	static int m_isblank(int c) { return c == 0x20 || c == '\t'; }

protected:
	int (*m_posixfun)(int);
	int m_bRightLeft;
	int m_bYes;
};

#endif//_POSIX_ELX_T_H__
