// PreBuilder.h
//
// Copyright 2006 (c) RegExLab.com
//
// Author: ʷΰ (sswater shi)
//
// 2006/07/11 08:47:01
//

#ifndef _PRE_BUILDER_H__
#define _PRE_BUILDER_H__

#include <list>

#include "RegexContext.h"
#include "ElxBuilderInterface.h"

#include "ZeroElx.h"
#include "ElxInterface.h"
#include "DelegateElx.h"
#include "ListElx.h"
#include "AlternativeElx.h"
#include "GreedyElx.h"
#include "AssertElx.h"
#include "AbsoluteGreedyElx.h"
#include "ReluctantElx.h"
#include "FixedRepeatElx.h"
#include "StringRef.h"
#include "FixedStringElx.h"
#include "BackrefElx.h"
#include "BoundaryElx.h"
#include "RangeElx.h"
#include "IndependentElx.h"
#include "BackrefFromElxT.h"
#include "GlobalPointElx.h"
#include "PosixElx.h"

template <typename CHART> class CMiniBufferT
{
public:
	CMiniBufferT()
	{
		m_pBuffer     = 0;
		m_nBufSize    = 0;
		m_nSize       = 0;
	}

	~CMiniBufferT()
	{
		if(m_pBuffer != 0) delete [] m_pBuffer;
	}

public:
	CMiniBufferT <CHART> & Append(CHART ch)
	{
		if((m_nSize + 2) > m_nBufSize)
		{
			int nNewSize = m_nBufSize * 2;
			if( nNewSize < 8 ) nNewSize = 8;

			CHART * pTemp = new CHART[nNewSize];

			if(m_pBuffer)
			{
				memcpy(pTemp, m_pBuffer, m_nBufSize * sizeof(CHART));
				delete [] m_pBuffer;
			}

			m_pBuffer   = pTemp;
			m_nBufSize  = nNewSize;
		}

		m_pBuffer[m_nSize++] = ch;
		m_pBuffer[m_nSize  ] = RCHART(0);

		return *this;
	}

	CHART * Detach()
	{
		CHART * pTemp = m_pBuffer;
		m_pBuffer     = 0;
		m_nBufSize    = 0;
		m_nSize       = 0;

		return pTemp;
	}

	void Release()
	{
		// release
		if(m_pBuffer != 0) delete [] m_pBuffer;

		Detach();
	}

	inline CHART & operator [] (int nIndex) { return m_pBuffer[nIndex]; }
	inline CHART * GetBuffer() { static CHART empty[] = {RCHART(0)}; return m_pBuffer ? m_pBuffer : empty; }
	inline int     GetSize  () { return m_nSize  ; }

private:
	CHART * m_pBuffer;
	int     m_nBufSize, m_nSize;
};

template <typename CHART> class CPreBuilderT : public ElxBuilderInterfaceT <CHART>
{
// super char struct
public:
	struct SPCHART
	{
	public:
		int bVchar;
		CHART ch;
		unsigned int nPos, nLen;

	public:
		SPCHART(CHART c, int bV = 0, unsigned int p = 0, unsigned int l = 0)
		{
			ch     = c;
			bVchar = bV;
			nPos   = p;
			nLen   = l;
		}

	public:
		inline int operator ==(const CHART       c) { return ch == c && bVchar == 0; }
		inline int operator ==(const SPCHART & spc) { return ch == spc.ch && bVchar == spc.bVchar; }
		inline int operator !=(const CHART       c) { return ! operator == (c); }
		inline int operator !=(const SPCHART & spc) { return ! operator == (spc); }
	};

// stock elx id
public:
	enum STOCKELX_ID_DEFINES
	{
		STOCKELX_ZERO = 0,

		STOCKELX_WORD,
		STOCKELX_NOT_WORD,

		STOCKELX_DOT_ALL,
		STOCKELX_DOT_NOT_ALL,

		STOCKELX_SPACE,
		STOCKELX_NOT_SPACE,

		STOCKELX_DIGITAL,
		STOCKELX_NOT_DIGITAL,

		STOCKELX_WORD_RIGHTLEFT,
		STOCKELX_NOT_WORD_RIGHTLEFT,

		STOCKELX_DOT_ALL_RIGHTLEFT,
		STOCKELX_DOT_NOT_ALL_RIGHTLEFT,

		STOCKELX_SPACE_RIGHTLEFT,
		STOCKELX_NOT_SPACE_RIGHTLEFT,

		STOCKELX_DIGITAL_RIGHTLEFT,
		STOCKELX_NOT_DIGITAL_RIGHTLEFT,

		STOCKELX_COUNT
	};

// build and clear
public:
	ElxInterface * Build(const StringRefT <CHART> & pattern, int flags = 0)
	{
		int i;

		// save
		m_pPattern = &pattern;

		// init
		m_nNextBackref = 1;
		m_nNextPos     = 0;
		m_nCsetDepth   = 0;

		m_nflags = flags;

		m_bufRecursiveTo  .Release();
		m_bufRecursiveFrom.Release();

		for(i=0; i<3; i++) MoveNext();

		// recursive
		m_bufRecursiveTo.Append(0);

		// build
		m_pTopElx = BuildAlternative(flags);

		m_bufRecursiveTo[0] = m_pTopElx;

		// post-process recursive
		for(i=0; i<m_bufRecursiveFrom.GetSize(); i++)
		{
			CDelegateElx * pElxFrom = m_bufRecursiveFrom[i];

			if( pElxFrom->GetParam1() < m_bufRecursiveTo.GetSize())
			{
				pElxFrom->SetElxs(m_bufRecursiveTo[pElxFrom->GetParam1()]);
			}
		}

		// return
		return m_pTopElx;
	}

	void Clear()
	{
		std::list <ElxInterface *>::iterator iter;
		for(iter = m_objlist.begin(); iter != m_objlist.end(); iter ++)
		{
			delete (*iter);
		}

		m_objlist.clear();

		memset(m_pStockElxs, 0, sizeof(m_pStockElxs));
	}

	inline int GetMaxBackref() { return m_nNextBackref - 1; }
	inline ElxInterface * GetTopElx() { return m_pTopElx; }
	inline int GetFlags() { return m_nflags; }

// build
protected:
	ElxInterface * BuildAlternative(int valflags);
	ElxInterface * BuildList(int & flags);
	ElxInterface * BuildRepeat(int & flags);
	ElxInterface * BuildSimple(int & flags);
	ElxInterface * BuildCharset(int & flags);
	ElxInterface * BuildRecursive(int & flags);
	ElxInterface * BuildBoundary(int & flags);
	ElxInterface * BuildBackref(int & flags);

	ElxInterface * GetStockElx(int nStockId);

	// keep pointer for remove
	inline ElxInterface * Keep(ElxInterface * pElx) { m_objlist.insert(m_objlist.end(), pElx); return pElx; }

// Construct
public:
	CPreBuilderT() : current(0), next(0), next2(0)
	{
	}

	~CPreBuilderT()
	{
		Clear();
	}

// move to next super char
protected:
	SPCHART current, next, next2;
	unsigned int m_nNextPos;
	unsigned int m_nCsetDepth;

	void MoveNext();
	int  GetNext2();

	static unsigned int Hex2Int(const CHART * pcsz, int length, int & used);

// attribute
protected:
	const StringRefT <CHART> * m_pPattern;
	std::list <ElxInterface *> m_objlist;
	int m_nflags;
	ElxInterface * m_pTopElx;
	ElxInterface * m_pStockElxs[STOCKELX_COUNT];

	// for backref
	int m_nNextBackref;

	// for recursive
	CMiniBufferT <ElxInterface *> m_bufRecursiveTo;
	CMiniBufferT <CDelegateElx *> m_bufRecursiveFrom;
};

//
// Alternative
//
template <typename CHART> ElxInterface * CPreBuilderT<CHART>::BuildAlternative(int valflags)
{
	if(current == RCHART(0))
		return 0;

	// flag instance
	int flags = valflags;

	// first part
	ElxInterface * pAlternativeOne = BuildList(flags);

	// check alternative
	if(current == SPCHART(RCHART('|'), 1))
	{
		CAlternativeElx * pAlternative = (CAlternativeElx *)Keep(new CAlternativeElx());
		pAlternative->AppendElx(pAlternativeOne);

		// loop
		while(current == SPCHART(RCHART('|'), 1))
		{
			// skip '|' itself
			MoveNext();

			pAlternativeOne = BuildList(flags);
			pAlternative->AppendElx(pAlternativeOne);
		}

		return pAlternative;
	}

	return pAlternativeOne;
}

//
// List
//
template <typename CHART> ElxInterface * CPreBuilderT<CHART>::BuildList(int & flags)
{
	if(current == RCHART(0) || current == SPCHART(RCHART('|'), 1) || current == SPCHART(RCHART(')'), 1))
		return GetStockElx(STOCKELX_ZERO);

	// first
	ElxInterface * pListOne = BuildRepeat(flags);

	if(current != RCHART(0) && current != SPCHART(RCHART('|'), 1) && current != SPCHART(RCHART(')'), 1))
	{
		CListElx * pList = (CListElx *)Keep(new CListElx());
		pList->AppendElx(pListOne);

		while(current != RCHART(0) && current != SPCHART(RCHART('|'), 1) && current != SPCHART(RCHART(')'), 1))
		{
			pListOne = BuildRepeat(flags);

			// add
			if(flags & RIGHTTOLEFT)
				pList->InsertElx(pListOne);
			else
				pList->AppendElx(pListOne);
		}

		return pList;
	}

	return pListOne;
}

//
// Repeat
//
template <typename CHART> ElxInterface * CPreBuilderT<CHART>::BuildRepeat(int & flags)
{
	// simple
	ElxInterface * pSimple = BuildSimple(flags);

	if(current.bVchar == 0) return pSimple;

	// is quantifier or not
	int bIsQuantifier = 1;

	// quantifier range
	unsigned int nMin = 0, nMax = 0;

	switch(current.ch)
	{
	case RCHART('{'):
		{
			CMiniBufferT <char> re;

			// skip '{'
			MoveNext();

			// copy
			while(current != RCHART(0) && current != SPCHART(RCHART('}'), 1))
			{
				re.Append((char)current.ch);
				MoveNext();
			}

			// skip '}'
			MoveNext();

			// read
			char comma;
			int red = sscanf(re.GetBuffer(), "%u%1s%u", &nMin, &comma, &nMax);

			// check
			if(red  <=  1 ) nMax = nMin;
			if(red  ==  2 ) nMax = (unsigned int)(-1);
			if(nMax < nMin) nMax = nMin;
		}
		break;

	case RCHART('?'):
		nMin = 0;
		nMax = 1;

		// skip '?'
		MoveNext();
		break;

	case RCHART('*'):
		nMin = 0;
		nMax = (unsigned int)(-1);

		// skip '*'
		MoveNext();
		break;

	case RCHART('+'):
		nMin = 1;
		nMax = (unsigned int)(-1);

		// skip '+'
		MoveNext();
		break;

	default:
		bIsQuantifier = 0;
		break;
	}

	// do quantify
	if(bIsQuantifier)
	{
		// 0 times
		if(nMax == 0)
			return GetStockElx(STOCKELX_ZERO);

		// at least 0 times
		if(nMin == 0)
		{
			if(current == SPCHART(RCHART('?'), 1))
			{
				MoveNext();
				return Keep(new CReluctantElx(pSimple, nMax));
			}
			else if(current == SPCHART(RCHART('+'), 1))
			{
				MoveNext();
				return Keep(new CAbsoluteGreedyElx(pSimple, nMax));
			}
			else
			{
				return Keep(new CGreedyElx(pSimple, nMax));
			}
		}

		// fixed times
		if(nMin == nMax)
			return Keep(new CFixedRepeatElx(pSimple, nMin));

		// range times
		CListElx * pList = (CListElx *)Keep(new CListElx());
		pList->AppendElx(Keep(new CFixedRepeatElx(pSimple, nMin)));

		if(nMax > nMin)
		{
			if(current == SPCHART(RCHART('?'), 1))
			{
				MoveNext();
				pList->AppendElx(Keep(new CReluctantElx(pSimple, nMax - nMin)));
			}
			else if(current == SPCHART(RCHART('+'), 1))
			{
				MoveNext();
				pList->AppendElx(Keep(new CAbsoluteGreedyElx(pSimple, nMax - nMin)));
			}
			else
			{
				pList->AppendElx(Keep(new CGreedyElx(pSimple, nMax - nMin)));
			}
		}

		return pList;
	}

	return pSimple;
}

//
// Simple
//
template <typename CHART> ElxInterface * CPreBuilderT<CHART>::BuildSimple(int & flags)
{
	CMiniBufferT <CHART> fixed;

	while(current != RCHART(0))
	{
		if(current.bVchar == 0)
		{
			if(next == SPCHART(RCHART('{'), 1) || next == SPCHART(RCHART('?'), 1) || next == SPCHART(RCHART('*'), 1) || next == SPCHART(RCHART('+'), 1))
			{
				if(fixed.GetSize() == 0)
				{
					fixed.Append(current.ch);
					MoveNext();
				}

				break;
			}
			else
			{
				fixed.Append(current.ch);
				MoveNext();
			}
		}
		else if(current.bVchar == 1)
		{
			CHART vch = current.ch;

			// end of simple
			if(vch == RCHART(')') || vch == RCHART('|'))
				break;

			// has fixed already
			if(fixed.GetSize() > 0)
				break;

			// left parentheses
			if(vch == RCHART('('))
			{
				return BuildRecursive(flags);
			}

			// char set
			if( vch == RCHART('[') || vch == RCHART('.') || vch == RCHART('w') || vch == RCHART('W') ||
				vch == RCHART('s') || vch == RCHART('S') || vch == RCHART('d') || vch == RCHART('D')
			)
			{
				return BuildCharset(flags);
			}

			// boundary
			if( vch == RCHART('^') || vch == RCHART('$') || vch == RCHART('A') || vch == RCHART('Z') ||
				vch == RCHART('b') || vch == RCHART('B') || vch == RCHART('G') // vch == RCHART('<') || vch == RCHART('>')
			)
			{
				return BuildBoundary(flags);
			}

			// backref
			if(vch == RCHART('\\'))
			{
				return BuildBackref(flags);
			}

			// treat vchar as char
			fixed.Append(current.ch);
			MoveNext();
		}
	}

	if(fixed.GetSize() > 0)
		return Keep(new CFixedStringElxT <CHART> (fixed.GetBuffer(), fixed.GetSize(), flags & RIGHTTOLEFT, flags & IGNORECASE));
	else
		return GetStockElx(STOCKELX_ZERO);
}

//
// Recursive
//
template <typename CHART> ElxInterface * CPreBuilderT<CHART>::BuildRecursive(int & flags)
{
	ElxInterface * pElx = 0;

	// skip '('
	MoveNext();

	if(current == SPCHART(RCHART('?'), 1))
	{
		// skip '?'
		MoveNext();

		int bNegative = 0;

		switch(current.ch)
		{
		case RCHART('!'):
			bNegative = 1;

		case RCHART('='):
			MoveNext(); // skip '!' or '='
			pElx = Keep(new CAssertElx(BuildAlternative(flags & ~RIGHTTOLEFT), !bNegative));
			break;

		case RCHART('<'):
			MoveNext(); // skip '<'
			switch(current.ch)
			{
			case RCHART('!'):
				bNegative = 1;

			case RCHART('='):
				MoveNext(); // skip '!' or '='

			default: // treat '(?<xxx)' as '(?<=xxx)'
				pElx = Keep(new CAssertElx(BuildAlternative(flags | RIGHTTOLEFT), !bNegative));
				break;
			}
			break;

		case RCHART('>'):
			MoveNext(); // skip '>'
			pElx = Keep(new CIndependentElx(BuildAlternative(flags)));
			break;

		case RCHART('R'):
			{
				MoveNext(); // skip 'R'

				CMiniBufferT <char> rto;
				while(current.ch != RCHART(0) && current.ch != RCHART(')'))
				{
					rto.Append((char)current.ch);
					MoveNext();
				}

				unsigned int rtono = 0;
				sscanf(rto.GetBuffer(), "%u", &rtono);

				CDelegateElx * pDelegate = (CDelegateElx *)Keep(new CDelegateElx(0, 0, rtono));

				m_bufRecursiveFrom.Append(pDelegate);

				pElx = pDelegate;
			}
			break;

		default:
			{
				// flag
				int newflags = flags;
				while(current.ch != RCHART(0) && current.ch != RCHART(':') && current.ch != RCHART(')'))
				{
					int tochange = 0;

					switch(current.ch)
					{
					case RCHART('i'):
					case RCHART('I'):
						tochange = IGNORECASE;
						break;

					case RCHART('s'):
					case RCHART('S'):
						tochange = SINGLELINE;
						break;

					case RCHART('m'):
					case RCHART('M'):
						tochange = MULTILINE;
						break;

					case RCHART('g'):
					case RCHART('G'):
						tochange = GLOBAL;
						break;

					case RCHART('-'):
						bNegative = 1;
						break;
					}

					if(bNegative)
						newflags &= ~tochange;
					else
						newflags |=  tochange;

					// move to next char
					MoveNext();
				}

				if(current.ch == RCHART(':'))
				{
					// skip ':'
					MoveNext();

					pElx = BuildAlternative(newflags);
				}
				else
				{
					// change parent flags
					flags = newflags;

					pElx = GetStockElx(STOCKELX_ZERO);
				}
			}
			break;
		}
	}
	else
	{
		// backref
		CListElx * pList = (CListElx *)Keep(new CListElx());

		int nThisBackref = m_nNextBackref ++;

		// left
		pList->AppendElx(Keep(new CBackrefElx(nThisBackref, 0)));

		// for recursive
		m_bufRecursiveTo.Append(0);

		pElx = BuildAlternative(flags);
		pList->AppendElx(pElx);

		m_bufRecursiveTo[nThisBackref] = pElx;

		// right
		pList->AppendElx(Keep(new CBackrefElx(nThisBackref, 1)));

		pElx = pList;
	}

	MoveNext(); // skip ')'

	return pElx;
}

//
// Boundary
//
template <typename CHART> ElxInterface * CPreBuilderT<CHART>::BuildBoundary(int & flags)
{
	// char
	CHART ch = current.ch;

	// skip
	MoveNext();

	switch(ch)
	{
	case RCHART('^'):
		return Keep(new CBoundaryElxT <CHART> ((flags & MULTILINE) ? LINE_BEGIN : FILE_BEGIN));

	case RCHART('$'):
		return Keep(new CBoundaryElxT <CHART> ((flags & MULTILINE) ? LINE_END : FILE_END));

	case RCHART('b'):
		return Keep(new CBoundaryElxT <CHART> (WORD_EDGE));

	case RCHART('B'):
		return Keep(new CBoundaryElxT <CHART> (WORD_EDGE, 0));

	case RCHART('A'):
		return Keep(new CBoundaryElxT <CHART> (FILE_BEGIN));

	case RCHART('Z'):
		return Keep(new CBoundaryElxT <CHART> (FILE_END));

	case RCHART('G'):
		if(flags & GLOBAL)
			return Keep(new CGlobalPointElx());
		else
			return GetStockElx(STOCKELX_ZERO);

	default:
		return GetStockElx(STOCKELX_ZERO);
	}
}

//
// Charset
//
template <typename CHART> ElxInterface * CPreBuilderT<CHART>::BuildCharset(int & flags)
{
	// char
	CHART ch = current.ch;

	// skip
	MoveNext();

	switch(ch)
	{
	case RCHART('.'):
		return GetStockElx(
			flags & RIGHTTOLEFT ?
			((flags & SINGLELINE) ? STOCKELX_DOT_ALL_RIGHTLEFT : STOCKELX_DOT_NOT_ALL_RIGHTLEFT) :
			((flags & SINGLELINE) ? STOCKELX_DOT_ALL : STOCKELX_DOT_NOT_ALL)
		);

	case RCHART('w'):
		return GetStockElx(flags & RIGHTTOLEFT ? STOCKELX_WORD_RIGHTLEFT : STOCKELX_WORD);

	case RCHART('W'):
		return GetStockElx(flags & RIGHTTOLEFT ? STOCKELX_NOT_WORD_RIGHTLEFT : STOCKELX_NOT_WORD);

	case RCHART('s'):
		return GetStockElx(flags & RIGHTTOLEFT ? STOCKELX_SPACE_RIGHTLEFT : STOCKELX_SPACE);

	case RCHART('S'):
		return GetStockElx(flags & RIGHTTOLEFT ? STOCKELX_NOT_SPACE_RIGHTLEFT : STOCKELX_NOT_SPACE);

	case RCHART('d'):
		return GetStockElx(flags & RIGHTTOLEFT ? STOCKELX_DIGITAL_RIGHTLEFT : STOCKELX_DIGITAL);

	case RCHART('D'):
		return GetStockElx(flags & RIGHTTOLEFT ? STOCKELX_NOT_DIGITAL_RIGHTLEFT : STOCKELX_NOT_DIGITAL);

	case RCHART('['):
		{
			CRangeElxT <CHART> * pRange = 0;

			// create
			if(current == SPCHART(RCHART(':'), 1))
			{
				CMiniBufferT <char> posix;

				do {
					posix.Append((char)current.ch);
					MoveNext();
				}
				while(current.ch != RCHART(0) && current != SPCHART(RCHART(']'), 1));

				posix.Append('\0');
				MoveNext(); // skip ']'

				// posix
				return Keep(new CPosixElxT <CHART> (posix.GetBuffer(), flags & RIGHTTOLEFT));
			}
			else if(current == SPCHART(RCHART('^'), 1))
			{
				MoveNext(); // skip '^'
				pRange = (CRangeElxT <CHART> *)Keep(new CRangeElxT <CHART> (flags & RIGHTTOLEFT, 0));
			}
			else
			{
				pRange = (CRangeElxT <CHART> *)Keep(new CRangeElxT <CHART> (flags & RIGHTTOLEFT, 1));
			}

			// parse
			while(current.ch != RCHART(0) && current != SPCHART(RCHART(']'), 1))
			{
				ch = current.ch;

				if(current.bVchar == 1 && (
					ch == RCHART('[') || ch == RCHART('.') || ch == RCHART('w') || ch == RCHART('W') ||
					ch == RCHART('s') || ch == RCHART('S') || ch == RCHART('d') || ch == RCHART('D')
				))
				{
					pRange->AppendEmbed(BuildCharset(flags));
				}
				else if(next == SPCHART(RCHART('-'), 1) && next2.bVchar == 0)
				{
					pRange->AppendRange(ch, next2.ch);

					// next
					MoveNext();
					MoveNext();
					MoveNext();
				}
				else
				{
					pRange->AppendChar(ch);

					// next
					MoveNext();
				}
			}

			// skip ']'
			MoveNext();

			return pRange;
		}
	}

	return GetStockElx(STOCKELX_ZERO);
}

//
// Backref
//
template <typename CHART> ElxInterface * CPreBuilderT<CHART>::BuildBackref(int & flags)
{
	// skip '\\'
	MoveNext();

	unsigned int nbackref = 0;

	for(int i=0; i<3; i++)
	{
		if(current.ch >= RCHART('0') && current.ch <= RCHART('9'))
			nbackref = nbackref * 10 + (current.ch - RCHART('0'));
		else
			break;

		MoveNext();
	}

	return Keep(new CBackrefFromElxT <CHART> (nbackref, flags & RIGHTTOLEFT, flags & IGNORECASE));
}

//
// hex to int
//
template <typename CHART> unsigned int CPreBuilderT<CHART>::Hex2Int(const CHART * pcsz, int length, int & used)
{
	unsigned int result = 0;
	int & i = used;

	for(i=0; i<length; i++)
	{
		if(pcsz[i] >= RCHART('0') && pcsz[i] <= RCHART('9'))
			result = (result << 4) + (pcsz[i] - RCHART('0'));
		else if(pcsz[i] >= RCHART('A') && pcsz[i] <= RCHART('F'))
			result = (result << 4) + (0x0A + (pcsz[i] - RCHART('A')));
		else if(pcsz[i] >= RCHART('a') && pcsz[i] <= RCHART('f'))
			result = (result << 4) + (0x0A + (pcsz[i] - RCHART('a')));
		else
			break;
	}

	return result;
}

//
// move to next char
//
template <typename CHART> void CPreBuilderT<CHART>::MoveNext()
{
	// move forwards
	current = next;
	next    = next2;

	// get next2
	int bgot = 0;
	while( !bgot ) bgot = GetNext2();
}

//
// get next2 char
//
template <typename CHART> int CPreBuilderT<CHART>::GetNext2()
{
	int delta = 1;
	next2     = SPCHART(0, 0, 0, 0);

	CHART ch  = m_pPattern->At(m_nNextPos);

	switch(ch)
	{
	case RCHART('\\'):
		{
			CHART ch1 = m_pPattern->At(m_nNextPos+1);

			// backref
			if(ch1 >= RCHART('0') && ch1 <= RCHART('9'))
			{
				next2 = SPCHART(ch, 1, m_nNextPos, delta);
				break;
			}

			// escape
			delta     = 2;

			switch(ch1)
			{
			case RCHART('A'):
			case RCHART('Z'):
			case RCHART('w'):
			case RCHART('W'):
			case RCHART('s'):
			case RCHART('S'):
			case RCHART('B'):
			case RCHART('d'):
			case RCHART('D'):
				next2 = SPCHART(ch1, 1, m_nNextPos, delta);
				break;

			case RCHART('b'):
				if(m_nCsetDepth > 0)
					next2 = SPCHART('\b', 0, m_nNextPos, delta);
				else
					next2 = SPCHART(ch1, 1, m_nNextPos, delta);
				break;

			/*
			case RCHART('<'):
			case RCHART('>'):
				if(m_nCsetDepth > 0)
					next2 = SPCHART(ch1, 0, m_nNextPos, delta);
				else
					next2 = SPCHART(ch1, 1, m_nNextPos, delta);
				break;
			*/

			case RCHART('x'):
				{
					int red = 0;
					unsigned int ch2 = Hex2Int(((const CHART *)(*m_pPattern)) + m_nNextPos + 2, 2, red);

					delta += red;

					if(red > 0)
						next2 = SPCHART(RCHART(ch2), 0, m_nNextPos, delta);
					else
						next2 = SPCHART(ch1, 0, m_nNextPos, delta);
				}
				break;

			case RCHART('u'):
				{
					int red = 0;
					unsigned int ch2 = Hex2Int(((const CHART *)(*m_pPattern)) + m_nNextPos + 2, 4, red);

					delta += red;

					if(red > 0)
						next2 = SPCHART(RCHART(ch2), 0, m_nNextPos, delta);
					else
						next2 = SPCHART(ch1, 0, m_nNextPos, delta);
				}
				break;

			case RCHART('a'): next2 = SPCHART(RCHART('\a'), 0, m_nNextPos, delta); break;
			case RCHART('f'): next2 = SPCHART(RCHART('\f'), 0, m_nNextPos, delta); break;
			case RCHART('n'): next2 = SPCHART(RCHART('\n'), 0, m_nNextPos, delta); break;
			case RCHART('r'): next2 = SPCHART(RCHART('\r'), 0, m_nNextPos, delta); break;
			case RCHART('t'): next2 = SPCHART(RCHART('\t'), 0, m_nNextPos, delta); break;
			case RCHART('v'): next2 = SPCHART(RCHART('\v'), 0, m_nNextPos, delta); break;
			case RCHART('e'): next2 = SPCHART(RCHART( 27 ), 0, m_nNextPos, delta); break;

			case RCHART('G'):  // skip '\G'
				if(m_nCsetDepth > 0)
				{
					m_nNextPos += 2;
					return 0;
				}
				else
				{
					next2 = SPCHART(ch1, 1, m_nNextPos, delta);
					break;
				}

			default:
				next2 = SPCHART(ch1, 0, m_nNextPos, delta);
				break;
			}
		}
		break;

	case RCHART('*'):
	case RCHART('+'):
	case RCHART('?'):
	case RCHART('.'):
	case RCHART('{'):
	case RCHART('}'):
	case RCHART(')'):
	case RCHART('|'):
	case RCHART('$'):
		if(m_nCsetDepth > 0)
			next2 = SPCHART(ch, 0, m_nNextPos, delta);
		else
			next2 = SPCHART(ch, 1, m_nNextPos, delta);
		break;

	case RCHART('^'):
		next2 = SPCHART(ch, 1, m_nNextPos, delta);
		break;

	case RCHART('-'):
		if(m_nCsetDepth > 0)
			next2 = SPCHART(ch, 1, m_nNextPos, delta);
		else
			next2 = SPCHART(ch, 0, m_nNextPos, delta);
		break;

	case RCHART('('):
		if(m_nCsetDepth > 0)
		{
			next2 = SPCHART(ch, 0, m_nNextPos, delta);
		}
		else
		{
			CHART ch1 = m_pPattern->At(m_nNextPos+1);
			CHART ch2 = m_pPattern->At(m_nNextPos+2);

			// skip remark
			if(ch1 == RCHART('?') && ch2 == RCHART('#'))
			{
				m_nNextPos += 2;
				while(m_nNextPos < m_pPattern->GetLength())
				{
					if(m_pPattern->At(m_nNextPos) == RCHART(')'))
						break;

					m_nNextPos ++;
				}

				if(m_pPattern->At(m_nNextPos) == RCHART(')'))
				{
					m_nNextPos ++;

					// get next next2
					return 0;
				}
			}
			else
			{
				next2 = SPCHART(ch, 1, m_nNextPos, delta);
			}
		}
		break;

	case RCHART('#'):
		if(m_nflags & EXTENDED)
		{
			// skip remark
			m_nNextPos ++;

			while(m_nNextPos < m_pPattern->GetLength())
			{
				if(m_pPattern->At(m_nNextPos) == RCHART('\n') || m_pPattern->At(m_nNextPos) == RCHART('\r'))
					break;

				m_nNextPos ++;
			}

			// get next next2
			return 0;
		}
		else
		{
			next2 = SPCHART(ch, 0, m_nNextPos, delta);
		}
		break;

	case RCHART(' '):
	case RCHART('\f'):
	case RCHART('\n'):
	case RCHART('\r'):
	case RCHART('\t'):
	case RCHART('\v'):
		if(m_nflags & EXTENDED)
		{
			m_nNextPos ++;

			// get next next2
			return 0;
		}
		else
		{
			next2 = SPCHART(ch, 0, m_nNextPos, delta);
		}
		break;

	case RCHART('['):
		m_nCsetDepth ++;
		next2 = SPCHART(ch, 1, m_nNextPos, delta);
		break;

	case RCHART(']'):
		m_nCsetDepth --;
		next2 = SPCHART(ch, 1, m_nNextPos, delta);
		break;

	case RCHART(':'):
		if(next == SPCHART(RCHART('['), 1))
			next2 = SPCHART(ch, 1, m_nNextPos, delta);
		else
			next2 = SPCHART(ch, 0, m_nNextPos, delta);
		break;

	default:
		next2 = SPCHART(ch, 0, m_nNextPos, delta);
		break;
	}

	m_nNextPos += delta;

	return 1;
}

//
// Stock
//
template <typename CHART> ElxInterface * CPreBuilderT<CHART>::GetStockElx(int nStockId)
{
	ElxInterface ** pStockElxs = m_pStockElxs;

	// check
	if(nStockId < 0 || nStockId >= STOCKELX_COUNT)
		return GetStockElx(0);

	// create if no
	if(pStockElxs[nStockId] == 0)
	{
		switch(nStockId)
		{
		case STOCKELX_ZERO:
			pStockElxs[nStockId] = Keep(new CZeroElx());
			break;

		case STOCKELX_WORD:
			{
				CRangeElxT <CHART> * pRange = (CRangeElxT <CHART> *)Keep(new CRangeElxT <CHART> (0, 1));

				pRange->AppendRange(RCHART('A'), RCHART('Z'));
				pRange->AppendRange(RCHART('a'), RCHART('z'));
				pRange->AppendRange(RCHART('0'), RCHART('9'));
				pRange->AppendChar (RCHART('_'));

				pStockElxs[nStockId] = pRange;
			}
			break;

		case STOCKELX_NOT_WORD:
			{
				CRangeElxT <CHART> * pRange = (CRangeElxT <CHART> *)Keep(new CRangeElxT <CHART> (0, 0));

				pRange->AppendRange(RCHART('A'), RCHART('Z'));
				pRange->AppendRange(RCHART('a'), RCHART('z'));
				pRange->AppendRange(RCHART('0'), RCHART('9'));
				pRange->AppendChar (RCHART('_'));

				pStockElxs[nStockId] = pRange;
			}
			break;

		case STOCKELX_DOT_ALL:
			pStockElxs[nStockId] = Keep(new CRangeElxT <CHART> (0, 0));
			break;

		case STOCKELX_DOT_NOT_ALL:
			{
				CRangeElxT <CHART> * pRange = (CRangeElxT <CHART> *)Keep(new CRangeElxT <CHART> (0, 0));

				pRange->AppendChar (RCHART('\n'));

				pStockElxs[nStockId] = pRange;
			}
			break;

		case STOCKELX_SPACE:
			{
				CRangeElxT <CHART> * pRange = (CRangeElxT <CHART> *)Keep(new CRangeElxT <CHART> (0, 1));

				pRange->AppendChar(RCHART(' '));
				pRange->AppendChar(RCHART('\t'));
				pRange->AppendChar(RCHART('\r'));
				pRange->AppendChar(RCHART('\n'));

				pStockElxs[nStockId] = pRange;
			}
			break;

		case STOCKELX_NOT_SPACE:
			{
				CRangeElxT <CHART> * pRange = (CRangeElxT <CHART> *)Keep(new CRangeElxT <CHART> (0, 0));

				pRange->AppendChar(RCHART(' '));
				pRange->AppendChar(RCHART('\t'));
				pRange->AppendChar(RCHART('\r'));
				pRange->AppendChar(RCHART('\n'));

				pStockElxs[nStockId] = pRange;
			}
			break;

		case STOCKELX_DIGITAL:
			{
				CRangeElxT <CHART> * pRange = (CRangeElxT <CHART> *)Keep(new CRangeElxT <CHART> (0, 1));

				pRange->AppendRange(RCHART('0'), RCHART('9'));

				pStockElxs[nStockId] = pRange;
			}
			break;

		case STOCKELX_NOT_DIGITAL:
			{
				CRangeElxT <CHART> * pRange = (CRangeElxT <CHART> *)Keep(new CRangeElxT <CHART> (0, 0));

				pRange->AppendRange(RCHART('0'), RCHART('9'));

				pStockElxs[nStockId] = pRange;
			}
			break;

		case STOCKELX_WORD_RIGHTLEFT:
			{
				CRangeElxT <CHART> * pRange = (CRangeElxT <CHART> *)Keep(new CRangeElxT <CHART> (1, 1));

				pRange->AppendRange(RCHART('A'), RCHART('Z'));
				pRange->AppendRange(RCHART('a'), RCHART('z'));
				pRange->AppendRange(RCHART('0'), RCHART('9'));
				pRange->AppendChar (RCHART('_'));

				pStockElxs[nStockId] = pRange;
			}
			break;

		case STOCKELX_NOT_WORD_RIGHTLEFT:
			{
				CRangeElxT <CHART> * pRange = (CRangeElxT <CHART> *)Keep(new CRangeElxT <CHART> (1, 0));

				pRange->AppendRange(RCHART('A'), RCHART('Z'));
				pRange->AppendRange(RCHART('a'), RCHART('z'));
				pRange->AppendRange(RCHART('0'), RCHART('9'));
				pRange->AppendChar (RCHART('_'));

				pStockElxs[nStockId] = pRange;
			}
			break;

		case STOCKELX_DOT_ALL_RIGHTLEFT:
			pStockElxs[nStockId] = Keep(new CRangeElxT <CHART> (1, 0));
			break;

		case STOCKELX_DOT_NOT_ALL_RIGHTLEFT:
			{
				CRangeElxT <CHART> * pRange = (CRangeElxT <CHART> *)Keep(new CRangeElxT <CHART> (1, 0));

				pRange->AppendChar (RCHART('\n'));

				pStockElxs[nStockId] = pRange;
			}
			break;

		case STOCKELX_SPACE_RIGHTLEFT:
			{
				CRangeElxT <CHART> * pRange = (CRangeElxT <CHART> *)Keep(new CRangeElxT <CHART> (1, 1));

				pRange->AppendChar(RCHART(' '));
				pRange->AppendChar(RCHART('\t'));
				pRange->AppendChar(RCHART('\r'));
				pRange->AppendChar(RCHART('\n'));
				pRange->AppendChar(RCHART('\f'));
				pRange->AppendChar(RCHART('\v'));

				pStockElxs[nStockId] = pRange;
			}
			break;

		case STOCKELX_NOT_SPACE_RIGHTLEFT:
			{
				CRangeElxT <CHART> * pRange = (CRangeElxT <CHART> *)Keep(new CRangeElxT <CHART> (1, 0));

				pRange->AppendChar(RCHART(' '));
				pRange->AppendChar(RCHART('\t'));
				pRange->AppendChar(RCHART('\r'));
				pRange->AppendChar(RCHART('\n'));
				pRange->AppendChar(RCHART('\f'));
				pRange->AppendChar(RCHART('\v'));

				pStockElxs[nStockId] = pRange;
			}
			break;

		case STOCKELX_DIGITAL_RIGHTLEFT:
			{
				CRangeElxT <CHART> * pRange = (CRangeElxT <CHART> *)Keep(new CRangeElxT <CHART> (1, 1));

				pRange->AppendRange(RCHART('0'), RCHART('9'));

				pStockElxs[nStockId] = pRange;
			}
			break;

		case STOCKELX_NOT_DIGITAL_RIGHTLEFT:
			{
				CRangeElxT <CHART> * pRange = (CRangeElxT <CHART> *)Keep(new CRangeElxT <CHART> (1, 0));

				pRange->AppendRange(RCHART('0'), RCHART('9'));

				pStockElxs[nStockId] = pRange;
			}
			break;
		}
	}

	// return
	return pStockElxs[nStockId];
}

#endif//_PRE_BUILDER_H__
