// RangeElx.h
//
// Copyright 2006 (c) RegExLab.com
//
// Author: ʷΰ (sswater shi)
//
// 2006/06/20 03:17:12
//

#ifndef _RANGE_ELX_H__
#define _RANGE_ELX_H__

#include <list>
#include "ElxInterface.h"

template <typename CHART> struct RangeT
{
	CHART cMin;
	CHART cMax;
};

template <typename CHART> class CRangeElxT : public ElxInterface
{
public:
	typedef RangeT <CHART> Range;

public:
	int Match(ContextInterface * pContext)
	{
		const CHART * pcsz   = (const CHART *)pContext->GetTarget();
		unsigned int  nBegin = pContext->GetCurrentPos();

		// check pos
		if( m_bRightLeft ? (nBegin == 0) : (nBegin >= pContext->GetLength()) )
			return 0;

		const CHART & pc     = pcsz[m_bRightLeft ? nBegin-1 : nBegin];
		int bsucc = 0;

		for(typename std::list <ElxInterface *> ::iterator e = m_embeds.begin(); !bsucc && e != m_embeds.end(); e++)
		{
			if((*e)->Match(pContext))
			{
				bsucc = 1;

				// move back pos
				pContext->SetCurrentPos(nBegin);

				break;
			}
		}

		for(typename std::list <Range> ::iterator r = m_ranges.begin(); !bsucc && r != m_ranges.end(); r++)
		{
			Range & range = *r;

			if(range.cMin <= pc && pc <= range.cMax)
			{
				bsucc = 1;
				break;
			}
		}

		for(typename std::list <CHART> ::iterator c = m_chars.begin(); !bsucc && c != m_chars.end(); c++)
		{
			if((*c) == pc)
			{
				bsucc = 1;
				break;
			}
		}

		bsucc = m_bYes ? bsucc : !bsucc;

		if(bsucc) pContext->SetCurrentPos(nBegin + (m_bRightLeft ? -1 : 1) );

		return bsucc;
	}

	int MatchNext(ContextInterface * pContext)
	{
		pContext->SetCurrentPos(pContext->GetCurrentPos() - (m_bRightLeft ? -1 : 1));
		return 0;
	}

public:
	CRangeElxT(int bRightLeft, int bYes)
	{
		m_bRightLeft = bRightLeft;
		m_bYes       = bYes;
	}

	inline CRangeElxT <CHART> & AppendEmbed(ElxInterface * em) { m_embeds.insert(m_embeds.end(), em); return *this; }
	inline CRangeElxT <CHART> & AppendRange(CHART cMin, CHART cMax) { Range r = {cMin, cMax}; m_ranges.insert(m_ranges.end(), r); return *this; }
	inline CRangeElxT <CHART> & AppendChar (CHART ch) { m_chars.insert(m_chars.end(), ch); return *this; }

protected:
	std::list <ElxInterface *> m_embeds;
	std::list <Range> m_ranges;
	std::list <CHART> m_chars;

	int m_bRightLeft;
	int m_bYes;
};

#endif//_RANGE_ELX_H__
