// RegexContext.cpp: implementation of the CRegexContext class.
//
// Copyright 2006 (c) RegExLab.com
//
// Author: ʷΰ (sswater shi)
//
// 2006/06/20 03:17:12
//

#include "RegexContext.h"
#include "Backref.h"

CRegexContext::CRegexContext()
{
	m_nCurrentPos   = 0;
	m_pMatchTarget  = 0;
	m_nTargetLength = 0;
	m_nGlobalPos    = 0;
	m_nLastLength   = 1;
}

CRegexContext::~CRegexContext()
{
	for(int i=0; i<m_vbackref.size(); i++)
	{
		CBackref * pbackref = m_vbackref.at(i);
		if(pbackref != 0)
			delete pbackref;
	}

	m_vbackref.clear();
}

void CRegexContext::Mark(ContextState & state)
{
	m_stack.Mark(state.stackstate);
	state.m_nCurrentPos = m_nCurrentPos;

	// push something
	m_stack.Push(0, !state.stackstate.nId);
}

void CRegexContext::Restore(ContextState state, int stackonly)
{
	m_stack.Restore(state.stackstate);

	if( ! stackonly )
		SetCurrentPos(state.m_nCurrentPos);
}

void CRegexContext::SetCurrentPos(unsigned int nCurrentPos)
{
	m_nCurrentPos = nCurrentPos;
}

void CRegexContext::SetGlobalPos(unsigned int nPos)
{
	m_nGlobalPos = nPos;
}

CBackref * CRegexContext::GetOrCreateAt(int nIndex)
{
	if(m_vbackref.size() <= nIndex)
	{
		for(int i=m_vbackref.size(); i<=nIndex; i++)
		{
			m_vbackref.push_back(0);
		}
	}

	if(m_vbackref.at(nIndex) == 0)
	{
		m_vbackref[nIndex] = new CBackref();
	}

	return m_vbackref.at(nIndex);
}

void CRegexContext::PushBackrefStart(int nbackref, unsigned int nStart, int bNoPush)
{
	GetOrCreateAt(nbackref)->PushStart(nStart, bNoPush);
}

void CRegexContext::PushBackrefEnd(int nbackref, unsigned int nEnd)
{
	CBackref * pbackref = (nbackref < m_vbackref.size()) ? m_vbackref.at(nbackref) : 0;
	if(pbackref != 0) pbackref->PushEnd(nEnd);

	m_lastParenMatch.Push(nbackref);
}

void CRegexContext::PopBackrefStart(int nbackref)
{
	CBackref * pbackref = (nbackref < m_vbackref.size()) ? m_vbackref.at(nbackref) : 0;
	if(pbackref != 0) pbackref->PopStart();
}

void CRegexContext::PopBackrefEnd(int nbackref)
{
	CBackref * pbackref = (nbackref < m_vbackref.size()) ? m_vbackref.at(nbackref) : 0;
	if(pbackref != 0) pbackref->PopEnd();

	unsigned int nb;
	m_lastParenMatch.Pop(nb);
}

int CRegexContext::GetBackref(int nbackref, unsigned int &nStart, unsigned int &nEnd)
{
	CBackref * pbackref = (nbackref < m_vbackref.size()) ? m_vbackref.at(nbackref) : 0;
	if(pbackref != 0)
		return pbackref->GetBackref(nStart, nEnd, m_nCurrentPos);
	else
		return 0;
}

int CRegexContext::GetLastParenMatch()
{
	unsigned int nb = 0;

	if(m_lastParenMatch.Peek(nb))
		return (int)nb;
	else
		return 0;
}

void CRegexContext::ResetAll(void *pTarget, int nLength)
{
	ResetStack();

	m_nCurrentPos   = 0;
	m_pMatchTarget  = pTarget;
	m_nTargetLength = nLength;
}

void CRegexContext::ResetStack()
{
	StackState state = {0, 0};
	m_stack.Restore(state);
	m_lastParenMatch.Restore(state);

	for(int i=0; i<m_vbackref.size(); i++)
	{
		CBackref * pbackref = m_vbackref.at(i);
		if(pbackref != 0)
			pbackref->Reset();
	}
}
