// StringRef.h
//
// Copyright 2006 (c) RegExLab.com
//
// Author: ʷΰ (sswater shi)
//
// 2006/06/20 03:17:12
//

#ifndef _STRING_REF_H__
#define _STRING_REF_H__

#include <memory.h>

#define RCHART(ch) ((CHART)ch)

template <typename CHART> class StringRefT
{
public:
	StringRefT(const CHART * pcsz, unsigned int length)
	{
		m_pConsts = pcsz;
		m_nLength = length;
	}
	StringRefT(const CHART * pcsz)
	{
		// count first
		int l = 0;
		while(pcsz[l]) l++;

		m_pConsts = pcsz;
		m_nLength = l;
	}
	virtual ~StringRefT() {};

	int nCompare(const CHART * pcsz) const
	{
		for(unsigned int i=0; i<m_nLength; i++)
		{
			int c = (int)m_pConsts[i] - (int)pcsz[i];
			if( c != 0 )
				return c;
		}

		return 0;
	}

	int nCompareNoCase(const CHART * pcsz) const
	{
		for(unsigned int i=0; i<m_nLength; i++)
		{
			int c = (int)m_pConsts[i] - (int)pcsz[i];
			if( c != 0 )
			{
				c = Upper(m_pConsts[i]) - Upper(pcsz[i]);
				if(c != 0)
					return c;
			}
		}

		return 0;
	}

public:
	inline const CHART At(int nIndex) const { return nIndex >= m_nLength ? 0 : m_pConsts[nIndex]; }
	inline const CHART & operator [](int nIndex) const { return m_pConsts[nIndex]; }
	inline int GetLength() const { return m_nLength; }
	inline operator const CHART *() const { return m_pConsts; }
	inline int operator ==(const CHART * pcsz) const { return ! nCompare(pcsz); }
	inline int operator ==(const StringRefT <CHART> & sreft) const { return ! nCompare(sreft.m_pConsts); }

protected:
	static inline CHART Upper(CHART ch)
	{
		if(ch >= RCHART('a') && ch <= RCHART('z'))
			return ch - ('a' - 'A');
		else
			return ch;
	}

protected:
	const CHART * m_pConsts;
	unsigned int  m_nLength;
};

template <typename CHART> class StringCopyT : public StringRefT <CHART>
{
public:
	StringCopyT(const CHART * pcsz, unsigned int length) : StringRefT <CHART> (0, length)
	{
		StringRefT<CHART>::m_pConsts = m_pString = new CHART[length + 1];
		memcpy(m_pString, pcsz, sizeof(CHART) * length);
		m_pString[length] = 0;
	}
	StringCopyT(CHART * psz) : StringRefT <CHART> (0, 0)
	{
		// count first
		int l = 0;
		while(psz[l]) l++;

		StringRefT<CHART>::m_pConsts = m_pString = psz;
		StringRefT<CHART>::m_nLength = l;
	}
	virtual ~StringCopyT()
	{
		delete [] m_pString;
	}

public:
	inline CHART & operator [](int nIndex) { return m_pString[nIndex]; }
	inline int operator ==(const CHART * pcsz) const { return ! StringRefT<CHART>::nCompare(pcsz); }
	inline int operator ==(const StringRefT <CHART> & sreft) const { return ! nCompare(sreft.m_pConsts); }
	inline int operator ==(const StringCopyT <CHART> & scpt) const { return ! nCompare(scpt.m_pConsts); }

protected:
	CHART * m_pString;
};

#endif//_STRING_REF_H__
