/*
 * deelx_c.h : declarations for deelx regex main class
 *
 * Copyright (c) RegExLab.com. All rights reserved.
 *
 * Author:
 *       Sswater Shi, 2008-1-30
 */

#ifndef __deelx__regex__main__struct__h__
#define __deelx__regex__main__struct__h__

#ifdef  __cplusplus
extern "C" {
#endif

enum deelx_regex_flags
{
	Singleline     = 0x01,
	Multiline      = 0x02,
	Global         = 0x04, /* not used */
	IgnoreCase     = 0x08,
	RightToLeft    = 0x10,
	Extended       = 0x20
};

typedef struct __regexp  { int _reserved; } * regexp_handle;
typedef struct __result  { int _reserved; } * result_handle;
typedef struct __context { int _reserved; } * context_handle;

/* general use */
regexp_handle  regexp_create          (int size_of_char);
void           regexp_compile         (regexp_handle regexp, void * szregexp, int nlength, int nflags);
result_handle  regexp_match           (regexp_handle regexp, void * szstring, int nlength, int nstart, result_handle result);
void           regexp_release_result  (result_handle result);
void           regexp_release         (regexp_handle regexp);
int            result_ismatched       (result_handle result);
int            result_getstart        (result_handle result);
int            result_getend          (result_handle result);

/* advanced use */
result_handle  regexp_match_exact     (regexp_handle regexp, void * szstring, int nlength, result_handle result);
context_handle regexp_prepare         (regexp_handle regexp, void * szstring, int nlength, int nstart, context_handle context);
result_handle  regexp_match_context   (regexp_handle regexp, context_handle context, result_handle result);
void           regexp_release_context (context_handle context);
int            result_maxgroupnumber  (result_handle result);
int            result_getgroupstart   (result_handle result, int ngroupnumber);
int            result_getgroupend     (result_handle result, int ngroupnumber);

#ifdef  __cplusplus
}
#endif

#endif/*__deelx__regex__main__struct__h__*/
