/*
 * deelx_ca.c
 *
 * Copyright (c) RegExLab.com. All rights reserved.
 *
 * Author:
 *       Sswater Shi, 2008-1-30
 */

typedef struct sbufferrefdata
{
	void * m_pT;
	int    m_nSize;

} SBufferRefData, * PBufferRefData;

typedef void (*VfnSetSize)(void * pData, int nSize);
typedef struct sbufferdata
{
	void * m_pT;
	int    m_nSize;
	int    m_nMaxLength;
	int    m_nSizeOfT;
	VfnSetSize m_fnSetSize;

} SBufferData, * PBufferData;

typedef SBufferData SFixedBufData, * PFixedBufData;
typedef SBufferData SStringBufData, * PStringBufData;

typedef void * PStringRefData;
typedef int (*FnStringCompare)(PStringRefData pData1, PStringRefData pData2);

typedef struct scapturestacdata
{
	int m_number;
	int m_startpos;
	int m_endpos;
	int m_zindex;

} SCaptureStackData, * PCaptureStackData;

typedef struct smatchcontext
{
	SBufferData    m_stack;
	SBufferData    m_capturestack;
	SBufferData    m_captureindex;
	SBufferRefData m_string;
	int            m_nCurrentPos;
	int            m_nBeginPos;
	int            m_nLastBeginPos;
	int            m_nParenZindex;

} SMatchContext, * PMatchContext;

typedef int (*VfnMatch)     (void * pElx, PMatchContext pContext);
typedef int (*VfnElxRelease)(void * pElx);
typedef struct selxinterface
{
	VfnMatch      m_fnMatch;
	VfnMatch      m_fnMatchNext;
	VfnElxRelease m_fnRelease;

} SElxInterface, * PElxInterface;

typedef struct selxalternative
{
	SElxInterface m_base;
	SBufferData   m_elxlist;

} SElxAlternative, * PElxAlternative;

typedef struct selxassert
{
	SElxInterface m_base;
	PElxInterface m_pelx;
	int m_byes;

} SElxAssert, * PElxAssert;

typedef struct selxbackrefer
{
	SElxInterface   m_base;
	int             m_number;
	FnStringCompare m_compare;
	int             m_rightleft;
	SBufferData     m_name;

} SElxBackRefer, * PElxBackRefer;

enum deelx_boundary_type
{
	deelx_file_begin, /* begin of whole text  */
	deelx_file_end  , /* end of whole text    */
	deelx_file_end_n, /* end of whole text, or before newline at the end */
	deelx_line_begin, /* begin of line        */
	deelx_line_end  , /* end of line          */
	deelx_word_begin, /* begin of word        */
	deelx_word_end  , /* end of word          */
	deelx_word_edge , /* begin or end of word */
	deelx_this_begin  /* begin of this match  */
};

typedef int (*VfnComparePC)(void * pch1, int ch2);
typedef struct selxboundary
{
	SElxInterface m_base;
	int           m_type;
	int           m_byes;
	int           m_nSizeOfT;
	VfnComparePC  m_compare;

} SElxBoundary, * PElxBoundary;

typedef struct selxbracket
{
	SElxInterface m_base;
	int           m_right;
	int           m_number;
	SBufferData   m_name;

} SElxBracket, * PElxBracket;

typedef struct selxconditional
{
	SElxInterface m_base;
	int           m_number;
	SBufferData   m_name;
	PElxInterface m_pelxask;
	PElxInterface m_pelxyes;
	PElxInterface m_pelxno;

} SElxConditional, * PElxConditional;

typedef struct selxempty
{
	SElxInterface m_base;

} SElxEmpty, * PElxEmpty;

typedef struct selxrepeat
{
	SElxInterface m_base;
	PElxInterface m_pelx;
	int           m_nfixed;

} SElxRepeat, * PElxRepeat;

typedef struct selxgreedy
{
	SElxRepeat m_base;
	int        m_nvart;

} SElxGreedy, * PElxGreedy;

typedef struct selxindependent
{
	SElxInterface   m_base;
	PElxInterface   m_pelx;

} SElxIndependent, * PElxIndependent;

typedef struct selxlist
{
	SElxInterface m_base;
	SBufferData   m_elxlist;
	int           m_rightleft;

} SElxList, * PElxList;

typedef struct selxpossessive
{
	SElxGreedy m_base;

} SElxPossessive, * PElxPossessive;

typedef int (*VfnComparePP)(void * pch1, void * pch2);
typedef struct selxrange
{
	SElxInterface m_base;
	SBufferData   m_chars;
	SBufferData   m_ranges;
	SBufferData   m_embeds;
	int           m_rightleft;
	int           m_byes;
	VfnComparePP  m_compare;

} SElxRange, * PElxRange;

typedef struct selxrecursive
{
	SElxInterface m_base;
	PElxInterface m_pelx;
	int           m_number;
	SBufferData   m_name;

} SElxRecursive, * PElxRecursive;

typedef struct selxreluctant
{
	SElxRepeat m_base;
	int        m_nvart;

} SElxReluctant, * PElxReluctant;

typedef struct selxstring
{
	SElxInterface   m_base;
	SFixedBufData   m_string;
	FnStringCompare m_compare;
	int             m_rightleft;

} SElxString, * PElxString;

typedef struct scharinfo
{
	long ch;
	int  type;

} SCharInfo, * PCharInfo;

typedef int (*FnQuoteFunc)(int);
typedef long(*FnGetChar)(void*pBuffer, int at);
typedef void*(*FnLongToChar)(long*);
typedef struct sbuildcontext
{
	PElxInterface m_ptopelx;
	int m_nflags;
	int m_nmaxnumber;
	int m_nnextnamed;
	int m_ngroupcount;
	int m_nSizeOfT;

	SBufferData m_objlist;
	SBufferData m_grouplist;
	SBufferData m_recursivelist;
	SBufferData m_namedlist;
	SBufferData m_namedbackreflist;
	SBufferData m_namedconditionlist;

	SBufferRefData m_pattern;
	SCharInfo curr, next, nex2;
	int m_nnextpos;
	int m_ncharsetdepth;
	int m_bquoted;
	FnQuoteFunc m_quote_fun;
	FnStringCompare m_compare;
	FnGetChar m_getch;
	FnLongToChar m_ltochar;

} SBuildContext, * PBuildContext;

#include <stdlib.h>
#include <string.h>
#include <ctype.h>

int little_endian()
{
	int n = 1;
	return *(char*)&n;
}

PBufferRefData BufferRefData_New(void * pT, int nSize)
{
	PBufferRefData pData = (PBufferRefData)malloc(sizeof(SBufferRefData));
	pData->m_pT       = pT;
	pData->m_nSize    = nSize;
	return pData;
}

void BufferRefData_Delete(PBufferRefData pData)
{
	free( pData );
}

void * BufferRefData_GetAt(PBufferRefData pData, int nIndex, int nSizeOfT)
{
	return (char*)pData->m_pT + nSizeOfT * nIndex;
}

void * BufferData_PrepareInsert(PBufferData pData, int nPos, int nSize)
{
	int nOldSize = pData->m_nSize;
	(*pData->m_fnSetSize)(pData, nPos > pData->m_nSize ? nPos : pData->m_nSize + nSize);

	if( nPos < nOldSize )
	{
		void * from = (char*)pData->m_pT + pData->m_nSizeOfT * nPos;
		void * to   = (char*)pData->m_pT + pData->m_nSizeOfT * (nPos + nSize);
		memmove(to, from, pData->m_nSizeOfT * (nOldSize - nPos));
	}

	return (char*)pData->m_pT + pData->m_nSizeOfT * nPos;
}

void * BufferData_PrepareAppend(PBufferData pData, int nSize)
{
	return BufferData_PrepareInsert(pData, pData->m_nSize, nSize);
}

void BufferData_Append(PBufferData pData, void * pT, int nSize)
{
	memcpy(BufferData_PrepareAppend(pData, nSize), pT, pData->m_nSizeOfT * nSize);
}

void BufferData_SetSize(PBufferData pData, int nSize)
{
	(*pData->m_fnSetSize)(pData, nSize);
}

void BufferData_SetMaxLength(PBufferData pData, int nSize)
{
	if( nSize > pData->m_nMaxLength )
	{
		if( pData->m_nMaxLength < 8 )
			pData->m_nMaxLength = 8;

		if( nSize > pData->m_nMaxLength )
			pData->m_nMaxLength *= 2;

		if( nSize > pData->m_nMaxLength )
		{
			pData->m_nMaxLength  = nSize + 11;
			pData->m_nMaxLength -= pData->m_nMaxLength & 0x07;
		}

		pData->m_pT = realloc(pData->m_pT, pData->m_nSizeOfT * pData->m_nMaxLength);
	}
}

void BufferData_SetSize_Vfn(PBufferData pData, int nSize)
{
	BufferData_SetMaxLength(pData, nSize);
	pData->m_nSize = nSize;
}

void BufferData_Init(PBufferData pData, int nSizeOfT)
{
	memset(pData, 0, sizeof(SBufferData));
	pData->m_nSizeOfT  = nSizeOfT;
	pData->m_fnSetSize = (VfnSetSize)BufferData_SetSize_Vfn;
}

PBufferData BufferData_New(void * pT, int nSize, int nSizeOfT)
{
	PBufferData pData = (PBufferData)malloc(sizeof(SBufferData));
	BufferData_Init  (pData, nSizeOfT);
	BufferData_Append(pData, pT, nSize);
	return pData;
}

void * BufferData_Detach(PBufferData pData)
{
	void * pT = pData->m_pT;

	pData->m_pT         = 0;
	pData->m_nSize      = 0;
	pData->m_nMaxLength = 0;

	return pT;
}

void BufferData_Release(PBufferData pData)
{
	void * pT = BufferData_Detach(pData);
	if( pT ) free(pT);
}

void BufferData_Delete(PBufferData pData)
{
	BufferData_Release(pData);
	free( pData );
}

void BufferData_Attach(PBufferData pData, void * pT, int nSize)
{
	BufferData_Release(pData);

	pData->m_pT         = pT;
	pData->m_nMaxLength = nSize;

	(*pData->m_fnSetSize)(pData, nSize);
}

void * BufferData_GetAt(PBufferData pData, int nIndex)
{
	return (char*)pData->m_pT + pData->m_nSizeOfT * nIndex;
}

int BufferData_Pop(PBufferData pData, void * pT)
{
	if( pData->m_nSize > 0 )
	{
		memcpy(pT, BufferData_GetAt(pData, pData->m_nSize - 1), pData->m_nSizeOfT);
		(*pData->m_fnSetSize)(pData, pData->m_nSize - 1);
		return 1;
	}
	else
	{
		return 0;
	}
}

void BufferData_Insert(PBufferData pData, int nIndex, void * pT, int nSize)
{
	memcpy(BufferData_PrepareInsert(pData, nIndex, nSize), pT, pData->m_nSizeOfT * nSize);
}

void BufferData_Remove(PBufferData pData, int nIndex, int nSize)
{
	if( nIndex < pData->m_nSize )
	{
		if( nIndex + nSize >= pData->m_nSize )
		{
			(*pData->m_fnSetSize)(pData, nIndex);
		}
		else
		{
			memmove((char*)pData->m_pT + pData->m_nSizeOfT * nIndex, (char*)pData->m_pT + pData->m_nSizeOfT * (nIndex + nSize), pData->m_nSizeOfT * (pData->m_nSize - nIndex - nSize));
			(*pData->m_fnSetSize)(pData, pData->m_nSize - nSize);
		}
	}
}

void BufferData_ReplaceAt(PBufferData pData, int nPos, int nLength, void * pT, int nSize)
{
	if( nPos + nLength >= pData->m_nSize )
	{
		(*pData->m_fnSetSize)(pData, nPos + nSize);
	}
	else if( nLength < nSize )
	{
		void * from, * to;
		int mlen = pData->m_nSize - (nPos + nLength);
		(*pData->m_fnSetSize)(pData, pData->m_nSize + nSize - nLength);   /* ensure m_pT valid */
		from = (char*)pData->m_pT + pData->m_nSizeOfT * (nPos + nLength);
		to   = (char*)pData->m_pT + pData->m_nSizeOfT * (nPos + nSize);
		memmove(to, from, pData->m_nSizeOfT * mlen);
	}
	else if( nLength > nSize )
	{
		int mlen = pData->m_nSize - (nPos + nLength);
		void * from = (char*)pData->m_pT + pData->m_nSizeOfT * (nPos + nLength);
		void * to   = (char*)pData->m_pT + pData->m_nSizeOfT * (nPos + nSize);
		memmove(to, from, pData->m_nSizeOfT * mlen);
		(*pData->m_fnSetSize)(pData, pData->m_nSize + nSize - nLength);   /* protect data */
	}

	memcpy((char*)pData->m_pT + pData->m_nSizeOfT * nPos, pT, pData->m_nSizeOfT * nSize);
}

int BufferData_GetIntAt(PBufferData pData, int nIndex)
{
	return *(int*)BufferData_GetAt(pData, nIndex);
}

void * BufferData_GetPointerAt(PBufferData pData, int nIndex)
{
	return *(void**)BufferData_GetAt(pData, nIndex);
}

void BufferData_Prepare(PBufferData pData, int nIndex, int nFill)
{
	if(nIndex >= pData->m_nSize)
	{
		int oldSize = pData->m_nSize;
		BufferData_SetSize(pData, nIndex + 1);
		memset(BufferData_GetAt(pData, oldSize), nFill, pData->m_nSizeOfT * (nIndex + 1 - oldSize));
	}
}

void FixedBufData_SetSize_Vfn(PFixedBufData pData, int nSize)
{
	if(pData->m_nMaxLength != nSize && nSize > 0)
		pData->m_pT = realloc(pData->m_pT, pData->m_nSizeOfT * nSize);
	pData->m_nSize = pData->m_nMaxLength = nSize;
}

void FixedBufData_Init(PFixedBufData pData, int nSizeOfT)
{
	memset(pData, 0, sizeof(SFixedBufData));
	pData->m_nSizeOfT  = nSizeOfT;
	pData->m_fnSetSize = (VfnSetSize)FixedBufData_SetSize_Vfn;
}

PFixedBufData FixedBufData_New(void * pT, int nSize, int nSizeOfT)
{
	PFixedBufData pData = (PFixedBufData)malloc(sizeof(SFixedBufData));
	FixedBufData_Init(pData, nSizeOfT);
	BufferData_Append(pData, pT, nSize);
	return pData;
}

void StringBufData_SetSize_Vfn(PStringBufData pData, int nSize)
{
	BufferData_SetMaxLength(pData, nSize + 1);
	memset(BufferData_GetAt(pData, nSize), 0, pData->m_nSizeOfT);
	pData->m_nSize = nSize;
}

void StringBufData_Init(PStringBufData pData, int nSizeOfT)
{
	memset(pData, 0, sizeof(SStringBufData));
	pData->m_nSizeOfT  = nSizeOfT;
	pData->m_fnSetSize = (VfnSetSize)StringBufData_SetSize_Vfn;
	(*pData->m_fnSetSize)(pData, 0);
}

PStringBufData StringBufData_New(void * pT, int nSize, int nSizeOfT)
{
	PStringBufData pData = (PStringBufData)malloc(sizeof(SStringBufData));
	StringBufData_Init(pData, nSizeOfT);
	BufferData_Append (pData, pT, nSize);
	return pData;
}

int StringRefData_Compare1(PStringRefData pData1, PStringRefData pData2)
{
	char * str1 = (char*)((PBufferRefData)pData1)->m_pT,  * str2 = (char*)((PBufferRefData)pData2)->m_pT;
	int    len1 =        ((PBufferRefData)pData1)->m_nSize, len2 =        ((PBufferRefData)pData2)->m_nSize;

	if(len1 != len2)
		return 1;
	else
	{
		int i;
		for(i=0; i<len1; i++)
			if(str1[i] != str2[i])
				return 1;
		return 0;
	}
}

int StringRefData_Compare1I(PStringRefData pData1, PStringRefData pData2)
{
	char * str1 = (char*)((PBufferRefData)pData1)->m_pT,  * str2 = (char*)((PBufferRefData)pData2)->m_pT;
	int    len1 =        ((PBufferRefData)pData1)->m_nSize, len2 =        ((PBufferRefData)pData2)->m_nSize;

	if(len1 != len2)
		return 1;
	else
	{
		int i;
		for(i=0; i<len1; i++)
			if(str1[i] != str2[i])
				if(toupper(str1[i]) != toupper(str2[i]))
					return 1;
		return 0;
	}
}

int StringRefData_Compare2(PStringRefData pData1, PStringRefData pData2)
{
	short * str1 = (short*)((PBufferRefData)pData1)->m_pT,  * str2 = (short*)((PBufferRefData)pData2)->m_pT;
	int     len1 =         ((PBufferRefData)pData1)->m_nSize, len2 =         ((PBufferRefData)pData2)->m_nSize;

	if(len1 != len2)
		return 1;
	else
	{
		int i;
		for(i=0; i<len1; i++)
			if(str1[i] != str2[i])
				return 1;
		return 0;
	}
}

int StringRefData_Compare2I(PStringRefData pData1, PStringRefData pData2)
{
	short * str1 = (short*)((PBufferRefData)pData1)->m_pT,  * str2 = (short*)((PBufferRefData)pData2)->m_pT;
	int     len1 =         ((PBufferRefData)pData1)->m_nSize, len2 =         ((PBufferRefData)pData2)->m_nSize;

	if(len1 != len2)
		return 1;
	else
	{
		int i;
		for(i=0; i<len1; i++)
			if(str1[i] != str2[i])
				if(toupper(str1[i]) != toupper(str2[i]))
					return 1;
		return 0;
	}
}

int StringRefData_Compare4(PStringRefData pData1, PStringRefData pData2)
{
	long * str1 = (long*)((PBufferRefData)pData1)->m_pT,  * str2 = (long*)((PBufferRefData)pData2)->m_pT;
	int    len1 =        ((PBufferRefData)pData1)->m_nSize, len2 =        ((PBufferRefData)pData2)->m_nSize;

	if(len1 != len2)
		return 1;
	else
	{
		int i;
		for(i=0; i<len1; i++)
			if(str1[i] != str2[i])
				return 1;
		return 0;
	}
}

int StringRefData_Compare4I(PStringRefData pData1, PStringRefData pData2)
{
	long * str1 = (long*)((PBufferRefData)pData1)->m_pT,  * str2 = (long*)((PBufferRefData)pData2)->m_pT;
	int    len1 =        ((PBufferRefData)pData1)->m_nSize, len2 =        ((PBufferRefData)pData2)->m_nSize;

	if(len1 != len2)
		return 1;
	else
	{
		int i;
		for(i=0; i<len1; i++)
			if(str1[i] != str2[i])
				if(toupper(str1[i]) != toupper(str2[i]))
					return 1;
		return 0;
	}
}

void MatchContext_Init(PMatchContext pData)
{
	memset(pData, 0, sizeof(SMatchContext));
	BufferData_Init(&pData->m_stack       , sizeof(int));
	BufferData_Init(&pData->m_capturestack, sizeof(SCaptureStackData));
	BufferData_Init(&pData->m_captureindex, sizeof(int));
}

PMatchContext MatchContext_New()
{
	PMatchContext pData = (PMatchContext)malloc(sizeof(SMatchContext));
	MatchContext_Init(pData);
	return pData;
}

void MatchContext_Release(PMatchContext pData)
{
	BufferData_Release(&pData->m_captureindex);
	BufferData_Release(&pData->m_capturestack);
	BufferData_Release(&pData->m_stack);
}

void MatchContext_Delete(PMatchContext pData)
{
	MatchContext_Release(pData);
	free( pData );
}

PCaptureStackData MatchContext_GetCaptureData(PMatchContext pContext, int nNumber)
{
	int index;
	if(nNumber >= pContext->m_captureindex.m_nSize)
		index = -1;
	else
		index = BufferData_GetIntAt(&pContext->m_captureindex, nNumber);

	if(index >= pContext->m_capturestack.m_nSize)
		index = pContext->m_capturestack.m_nSize - 1;

	while(index >= 0)
	{
		PCaptureStackData pData = (PCaptureStackData)BufferData_GetAt(&pContext->m_capturestack, index);
		if(pData->m_number == nNumber)
			return pData;
		else
			index --;
	}

	return 0;
}

int ElxInterface_Match(void * pElx, PMatchContext pContext)
{
	return (*((PElxInterface)pElx)->m_fnMatch)(pElx, pContext);
}

int ElxInterface_MatchNext(void * pElx, PMatchContext pContext)
{
	if(((PElxInterface)pElx)->m_fnMatchNext)
		return (*((PElxInterface)pElx)->m_fnMatchNext)(pElx, pContext);
	else
		return 0;
}

void ElxInterface_Delete(void * pElx)
{
	if(((PElxInterface)pElx)->m_fnRelease)(*((PElxInterface)pElx)->m_fnRelease)(pElx);
	free( pElx );
}

int ElxAlternative_Match(PElxAlternative pData, PMatchContext pContext)
{
	int n;

	if(pData->m_elxlist.m_nSize == 0)
		return 1;

	for(n = 0; n < pData->m_elxlist.m_nSize; n++)
	{
		if(ElxInterface_Match(BufferData_GetPointerAt(&pData->m_elxlist, n), pContext))
		{
			BufferData_Append(&pContext->m_stack, &n, 1);
			return 1;
		}
	}

	return 0;
}

int ElxAlternative_MatchNext(PElxAlternative pData, PMatchContext pContext)
{
	int n = 0;

	if(pData->m_elxlist.m_nSize == 0)
		return 0;

	BufferData_Pop(&pContext->m_stack, &n);

	if(ElxInterface_MatchNext(BufferData_GetPointerAt(&pData->m_elxlist, n), pContext))
	{
		BufferData_Append(&pContext->m_stack, &n, 1);
		return 1;
	}
	else
	{
		for(n++; n < pData->m_elxlist.m_nSize; n++)
		{
			if(ElxInterface_Match(BufferData_GetPointerAt(&pData->m_elxlist, n), pContext))
			{
				BufferData_Append(&pContext->m_stack, &n, 1);
				return 1;
			}
		}

		return 0;
	}
}

void ElxAlternative_Release(PElxAlternative pData)
{
	BufferData_Release(&pData->m_elxlist);
}

PElxAlternative ElxAlternative_New()
{
	PElxAlternative pData = (PElxAlternative)malloc(sizeof(SElxAlternative));

	pData->m_base.m_fnMatch     = (VfnMatch)ElxAlternative_Match;
	pData->m_base.m_fnMatchNext = (VfnMatch)ElxAlternative_MatchNext;
	pData->m_base.m_fnRelease   = (VfnElxRelease)ElxAlternative_Release;

	BufferData_Init(&pData->m_elxlist, sizeof(PElxInterface));

	return pData;
}

int ElxAssert_Match(PElxAssert pData, PMatchContext pContext)
{
	int nbegin = pContext->m_nCurrentPos;
	int nsize  = pContext->m_stack.m_nSize;
	int ncsize = pContext->m_capturestack.m_nSize;

	int bsucc = ElxInterface_Match(pData->m_pelx, pContext);
	if( ! pData->m_byes ) bsucc = ! bsucc;

	BufferData_SetSize(&pContext->m_stack, nsize);
	pContext->m_nCurrentPos = nbegin;

	if( bsucc )
		BufferData_Append(&pContext->m_stack, &ncsize, 1);
	else
		BufferData_SetSize(&pContext->m_capturestack, ncsize);

	return bsucc;
}

int ElxAssert_MatchNext(PElxAssert pData, PMatchContext pContext)
{
	int ncsize = 0;

	BufferData_Pop(&pContext->m_stack, &ncsize);
	BufferData_SetSize(&pContext->m_capturestack, ncsize);

	return 0;
}

PElxAssert ElxAssert_New(PElxInterface pElx, int bYes)
{
	PElxAssert pData = (PElxAssert)malloc(sizeof(SElxAssert));

	pData->m_base.m_fnMatch     = (VfnMatch)ElxAssert_Match;
	pData->m_base.m_fnMatchNext = (VfnMatch)ElxAssert_MatchNext;
	pData->m_base.m_fnRelease   = 0;

	pData->m_pelx = pElx;
	pData->m_byes = bYes;

	return pData;
}

int ElxBackRefer_Match(PElxBackRefer pData, PMatchContext pContext)
{
	PCaptureStackData pCaptureData = MatchContext_GetCaptureData(pContext, pData->m_number);
	int left, right;
	SBufferRefData cap, ref;
	int bsucc;

	if( ! pCaptureData )
		return 0;

	left  = pCaptureData->m_startpos;
	right = pCaptureData->m_endpos;

	if(right < 0)
		right = pContext->m_nCurrentPos;

	if(right < left)
	{
		int t = right;
		right = left;
		left  = t;
	}

	cap.m_nSize = ref.m_nSize = right - left;
	cap.m_pT    = BufferRefData_GetAt(&pContext->m_string, left, pData->m_name.m_nSizeOfT);

	if(pData->m_rightleft)
	{
		if(pContext->m_nCurrentPos < ref.m_nSize)
			return 0;

		ref.m_pT = BufferRefData_GetAt(&pContext->m_string, pContext->m_nCurrentPos - ref.m_nSize, pData->m_name.m_nSizeOfT);
		bsucc = ! (*pData->m_compare)(&cap, &ref);

		if( bsucc )
		{
			BufferData_Append(&pContext->m_stack, &pContext->m_nCurrentPos, 1);
			pContext->m_nCurrentPos -= ref.m_nSize;
		}
	}
	else
	{
		if(pContext->m_nCurrentPos + ref.m_nSize > pContext->m_string.m_nSize)
			return 0;

		ref.m_pT = BufferRefData_GetAt(&pContext->m_string, pContext->m_nCurrentPos, pData->m_name.m_nSizeOfT);
		bsucc = ! (*pData->m_compare)(&cap, &ref);

		if( bsucc )
		{
			BufferData_Append(&pContext->m_stack, &pContext->m_nCurrentPos, 1);
			pContext->m_nCurrentPos += ref.m_nSize;
		}
	}

	return bsucc;
}

int ElxBackRefer_MatchNext(PElxBackRefer pData, PMatchContext pContext)
{
	BufferData_Pop(&pContext->m_stack, &pContext->m_nCurrentPos);

	return 0;
}

void ElxBackRefer_Release(PElxBackRefer pData)
{
	BufferData_Release(&pData->m_name);
}

PElxBackRefer ElxBackRefer_New(int nNumber, int nSizeOfT, int bRightLeft, int bIgnoreCase)
{
	PElxBackRefer pData = (PElxBackRefer)malloc(sizeof(SElxBackRefer));

	pData->m_base.m_fnMatch     = (VfnMatch)ElxBackRefer_Match;
	pData->m_base.m_fnMatchNext = (VfnMatch)ElxBackRefer_MatchNext;
	pData->m_base.m_fnRelease   = (VfnElxRelease)ElxBackRefer_Release;

	pData->m_number    = nNumber;
	pData->m_rightleft = bRightLeft;

	BufferData_Init(&pData->m_name, nSizeOfT);

	switch(nSizeOfT)
	{
	case 1: pData->m_compare = bIgnoreCase ? StringRefData_Compare1I : StringRefData_Compare1; break;
	case 2: pData->m_compare = bIgnoreCase ? StringRefData_Compare2I : StringRefData_Compare2; break;
	case 4: pData->m_compare = bIgnoreCase ? StringRefData_Compare4I : StringRefData_Compare4; break;
	default:pData->m_compare = 0;
	}

	return pData;
}

int ElxBoundary_WordChar(PElxBoundary pData, void * pch)
{
	return ((*pData->m_compare)(pch, 'A') >= 0 && (*pData->m_compare)(pch, 'Z') <= 0)
		|| ((*pData->m_compare)(pch, 'a') >= 0 && (*pData->m_compare)(pch, 'z') <= 0)
		|| ((*pData->m_compare)(pch, '0') >= 0 && (*pData->m_compare)(pch, '9') <= 0)
		|| ((*pData->m_compare)(pch, '_') == 0);
}

int ElxBoundary_Match(PElxBoundary pData, PMatchContext pContext)
{
	long zero = 0;
	int  npos = pContext->m_nCurrentPos;
	int  tlen = pContext->m_string.m_nSize;

	void * chL = npos <= 0    ? &zero : BufferRefData_GetAt(&pContext->m_string, npos - 1, pData->m_nSizeOfT);
	void * chR = npos >= tlen ? &zero : BufferRefData_GetAt(&pContext->m_string, npos    , pData->m_nSizeOfT);

	int bsucc;

	switch(pData->m_type)
	{
	case deelx_file_begin:
		bsucc = (npos <= 0);
		break;

	case deelx_file_end:
		bsucc = (npos >= tlen);
		break;

	case deelx_file_end_n:
		if(bsucc = (npos >= tlen) || (npos == tlen - 1 && ((*pData->m_compare)(chR , '\r') == 0 || (*pData->m_compare)(chR, '\n') == 0)))
		{
			break;
		}
		else if(npos == tlen - 2)
		{
			void * chR2 = BufferRefData_GetAt(&pContext->m_string, npos + 1, pData->m_nSizeOfT);
			bsucc = (*pData->m_compare)(chR, '\r') == 0 && (*pData->m_compare)(chR2, '\n') == 0;
		}
		break;

	case deelx_line_begin:
		bsucc = (npos <= 0) || (*pData->m_compare)(chL, '\n') == 0 || ((*pData->m_compare)(chL, '\r') == 0 && (*pData->m_compare)(chR, '\n') != 0);
		break;

	case deelx_line_end:
		bsucc = (npos >= tlen) || (*pData->m_compare)(chR, '\r') == 0 || ((*pData->m_compare)(chR, '\n') == 0 && (*pData->m_compare)(chL, '\r') != 0);
		break;

	case deelx_word_begin:
		bsucc = ! ElxBoundary_WordChar(pData, chL) &&   ElxBoundary_WordChar(pData, chR);
		break;

	case deelx_word_end:
		bsucc =   ElxBoundary_WordChar(pData, chL) && ! ElxBoundary_WordChar(pData, chR);
		break;

	case deelx_word_edge:
		bsucc =   ElxBoundary_WordChar(pData, chL) ?  ! ElxBoundary_WordChar(pData, chR) : ElxBoundary_WordChar(pData, chR);
		break;

	case deelx_this_begin:
		bsucc = npos == pContext->m_nBeginPos;
		break;
	}

	return pData->m_byes ? bsucc : ! bsucc;
}

int ElxBoundary_Compare1(void * pch1, int ch2)
{
	if(*(unsigned char*)pch1 < ch2)
		return -1;
	else if(*(unsigned char*)pch1 == ch2)
		return 0;
	else
		return 1;
}

int ElxBoundary_Compare2(void * pch1, int ch2)
{
	if(*(unsigned short*)pch1 < ch2)
		return -1;
	else if(*(unsigned short*)pch1 == ch2)
		return 0;
	else
		return 1;
}

int ElxBoundary_Compare4(void * pch1, int ch2)
{
	if(*(unsigned long*)pch1 < (unsigned)ch2)
		return -1;
	else if(*(unsigned long*)pch1 == (unsigned)ch2)
		return 0;
	else
		return 1;
}

PElxBoundary ElxBoundary_New(int nSizeOfT, int nType, int bYes)
{
	PElxBoundary pData = (PElxBoundary)malloc(sizeof(SElxBoundary));

	pData->m_base.m_fnMatch     = (VfnMatch)ElxBoundary_Match;
	pData->m_base.m_fnMatchNext = 0;
	pData->m_base.m_fnRelease   = 0;

	pData->m_type     = nType;
	pData->m_byes     = bYes;
	pData->m_nSizeOfT = nSizeOfT;

	switch(nSizeOfT)
	{
	case 1: pData->m_compare = ElxBoundary_Compare1; break;
	case 2: pData->m_compare = ElxBoundary_Compare2; break;
	case 4: pData->m_compare = ElxBoundary_Compare4; break;
	default:pData->m_compare = 0;
	}

	return pData;
}

int ElxBracket_Match(PElxBracket pData, PMatchContext pContext)
{
	PCaptureStackData pCaptureData;

	if(pData->m_number < 0)
		return 0;

	pCaptureData = MatchContext_GetCaptureData(pContext, pData->m_number);

	if( ! pData->m_right )
	{
		SCaptureStackData cData;

		if(pCaptureData && pCaptureData->m_endpos < 0)
		{
			pCaptureData->m_zindex --;
			return 1;
		}

		cData.m_number   = pData->m_number;
		cData.m_startpos = pContext->m_nCurrentPos;
		cData.m_endpos   = -1;
		cData.m_zindex   = 0;

		BufferData_Prepare(&pContext->m_captureindex, pData->m_number, -1);
		*(int*)BufferData_GetAt(&pContext->m_captureindex, pData->m_number) = pContext->m_capturestack.m_nSize;
		BufferData_Append(&pContext->m_capturestack, &cData, 1);
	}
	else
	{
		if(pCaptureData)
		{
			if(pCaptureData->m_zindex < 0)
			{
				pCaptureData->m_zindex ++;
				return 1;
			}

			pCaptureData->m_endpos = pContext->m_nCurrentPos;
			pCaptureData->m_zindex = pContext->m_nParenZindex ++;
		}
	}

	return 1;
}

int ElxBracket_MatchNext(PElxBracket pData, PMatchContext pContext)
{
	PCaptureStackData pCaptureData = MatchContext_GetCaptureData(pContext, pData->m_number);

	if( ! pCaptureData )
		return 0;

	if( ! pData->m_right )
	{
		if(pCaptureData->m_zindex < 0)
		{
			pCaptureData->m_zindex ++;
			return 0;
		}
		else
		{
			int index;

			BufferData_SetSize(&pContext->m_capturestack, pContext->m_capturestack.m_nSize - 1);

			/* fix index */
			for(index = pContext->m_capturestack.m_nSize - 1; index >= 0; index --)
				if(((PCaptureStackData)BufferData_GetAt(&pContext->m_capturestack, index))->m_number == pData->m_number)
					break;

			*(int*)BufferData_GetAt(&pContext->m_captureindex, pData->m_number) = index;
		}
	}
	else
	{
		if(pCaptureData->m_endpos >= 0)
		{
			pCaptureData->m_endpos = -1;
			pCaptureData->m_zindex = 0;
		}
		else
		{
			pCaptureData->m_zindex --;
		}
	}

	return 0;
}

void ElxBracket_Release(PElxBracket pData)
{
	BufferData_Release(&pData->m_name);
}

PElxBracket ElxBracket_New(int nNumber, int bRight, int nSizeOfT)
{
	PElxBracket pData = (PElxBracket)malloc(sizeof(SElxBracket));

	pData->m_base.m_fnMatch     = (VfnMatch)ElxBracket_Match;
	pData->m_base.m_fnMatchNext = (VfnMatch)ElxBracket_MatchNext;
	pData->m_base.m_fnRelease   = (VfnElxRelease)ElxBracket_Release;

	pData->m_number = nNumber;
	pData->m_right  = bRight;

	BufferData_Init(&pData->m_name, nSizeOfT);

	return pData;
}

int ElxConditional_Match(PElxConditional pData, PMatchContext pContext)
{
	int nbegin = pContext->m_nCurrentPos;
	int nsize  = pContext->m_stack.m_nSize;
	int ncsize = pContext->m_capturestack.m_nSize;

	int bsucc;
	int condition_yes = 0;

	if(pData->m_number >= 0)
	{
		condition_yes = MatchContext_GetCaptureData(pContext, pData->m_number) != 0;
	}
	else
	{
		if(pData->m_pelxask == 0)
			condition_yes = 1;
		else
			condition_yes = ElxInterface_Match(pData->m_pelxask, pContext);

		BufferData_SetSize(&pContext->m_stack, nsize);
		pContext->m_nCurrentPos = nbegin;
	}

	if(condition_yes)
		bsucc = pData->m_pelxyes == 0 ? 1 : ElxInterface_Match(pData->m_pelxyes, pContext);
	else
		bsucc = pData->m_pelxno  == 0 ? 1 : ElxInterface_Match(pData->m_pelxno, pContext);

	if( bsucc )
	{
		BufferData_Append(&pContext->m_stack, &ncsize, 1);
		BufferData_Append(&pContext->m_stack, &condition_yes, 1);
	}
	else
	{
		BufferData_SetSize(&pContext->m_capturestack, ncsize);
	}

	return bsucc;
}

int ElxConditional_MatchNext(PElxConditional pData, PMatchContext pContext)
{
	int ncsize, condition_yes;
	int bsucc;

	BufferData_Pop(&pContext->m_stack, &condition_yes);
	BufferData_Pop(&pContext->m_stack, &ncsize);

	if(condition_yes)
		bsucc = pData->m_pelxyes == 0 ? 0 : ElxInterface_MatchNext(pData->m_pelxyes, pContext);
	else
		bsucc = pData->m_pelxno  == 0 ? 0 : ElxInterface_MatchNext(pData->m_pelxno, pContext);

	if( bsucc )
	{
		BufferData_Append(&pContext->m_stack, &ncsize, 1);
		BufferData_Append(&pContext->m_stack, &condition_yes, 1);
	}
	else
	{
		BufferData_SetSize(&pContext->m_capturestack, ncsize);
	}

	return bsucc;
}

void ElxConditional_Release(PElxConditional pData)
{
	BufferData_Release(&pData->m_name);
}

PElxConditional ElxConditional_New(int nSizeOfT)
{
	PElxConditional pData = (PElxConditional)malloc(sizeof(SElxConditional));

	pData->m_base.m_fnMatch     = (VfnMatch)ElxConditional_Match;
	pData->m_base.m_fnMatchNext = (VfnMatch)ElxConditional_MatchNext;
	pData->m_base.m_fnRelease   = (VfnElxRelease)ElxConditional_Release;

	BufferData_Init(&pData->m_name, nSizeOfT);

	pData->m_number  = -1;
	pData->m_pelxask = 0;
	pData->m_pelxyes = 0;
	pData->m_pelxno  = 0;

	return pData;
}

int ElxEmpty_Match(PElxEmpty pData, PMatchContext pContext)
{
	return 1;
}

PElxEmpty ElxEmpty_New()
{
	PElxEmpty pData = (PElxEmpty)malloc(sizeof(SElxEmpty));

	pData->m_base.m_fnMatch     = (VfnMatch)ElxEmpty_Match;
	pData->m_base.m_fnMatchNext = 0;
	pData->m_base.m_fnRelease   = 0;

	return pData;
}

int ElxRepeat_Match(PElxRepeat pData, PMatchContext pContext)
{
	int n = 0;

	if(pData->m_nfixed == 0)
		return 1;

	while(n < pData->m_nfixed)
	{
		if(ElxInterface_Match(pData->m_pelx, pContext))
		{
			n ++;
		}
		else
		{
			n --;

			while(n >= 0 && ! ElxInterface_MatchNext(pData->m_pelx, pContext))
				n --;

			if(n >= 0)
				n ++;
			else
				return 0;
		}
	}

	return 1;
}

int ElxRepeat_MatchNext(PElxRepeat pData, PMatchContext pContext)
{
	int n = pData->m_nfixed - 1;

	if(pData->m_nfixed == 0)
		return 0;

	while(n >= 0 && ! ElxInterface_MatchNext(pData->m_pelx, pContext))
		n --;

	if(n >= 0)
		n ++;
	else
		return 0;

	while(n < pData->m_nfixed)
	{
		if(ElxInterface_Match(pData->m_pelx, pContext))
		{
			n ++;
		}
		else
		{
			n --;

			while(n >= 0 && ! ElxInterface_MatchNext(pData->m_pelx, pContext))
				n --;

			if(n >= 0)
				n ++;
			else
				return 0;
		}
	}

	return 1;
}

PElxRepeat ElxRepeat_New(PElxInterface pElx, int nFixed)
{
	PElxRepeat pData = (PElxRepeat)malloc(sizeof(SElxRepeat));

	pData->m_base.m_fnMatch     = (VfnMatch)ElxRepeat_Match;
	pData->m_base.m_fnMatchNext = (VfnMatch)ElxRepeat_MatchNext;
	pData->m_base.m_fnRelease   = 0;

	pData->m_pelx   = pElx;
	pData->m_nfixed = nFixed;

	return pData;
}

int ElxGreedy_MatchVart(PElxGreedy pData, PMatchContext pContext)
{
	int n      = 0;
	int nbegin = pContext->m_nCurrentPos;
	int nbegin00 = nbegin;

	while(n < pData->m_nvart && ElxInterface_Match(pData->m_base.m_pelx, pContext))
	{
		while(pContext->m_nCurrentPos == nbegin)
		{
			if( ! ElxInterface_MatchNext(pData->m_base.m_pelx, pContext) ) break;
		}

		if(pContext->m_nCurrentPos == nbegin) break;

		n ++;
		nbegin = pContext->m_nCurrentPos;
	}

	BufferData_Append(&pContext->m_stack, &nbegin00, 1);
	BufferData_Append(&pContext->m_stack, &n, 1);

	return 1;
}

int ElxGreedy_MatchNextVart(PElxGreedy pData, PMatchContext pContext)
{
	int n = 0, nbegin00 = 0;
	BufferData_Pop(&pContext->m_stack, &n);
	BufferData_Pop(&pContext->m_stack, &nbegin00);

	int nbegin0 = pContext->m_nCurrentPos;
	int n0 = n;

	if(n == 0) return 0;

	if( ! ElxInterface_MatchNext(pData->m_base.m_pelx, pContext) )
	{
		n --;
	}

	// not to re-match
	else if(pContext->m_nCurrentPos == nbegin00)
	{
		while(ElxInterface_MatchNext(pData->m_base.m_pelx, pContext)) { }
		n --;
	}

	// fix 2012-10-26, thanks to chenlx01@sohu.com
	else
	{
		int nbegin = pContext->m_nCurrentPos;
		while(n < pData->m_nvart && ElxInterface_Match(pData->m_base.m_pelx, pContext))
		{
			while(pContext->m_nCurrentPos == nbegin)
			{
				if( ! ElxInterface_MatchNext(pData->m_base.m_pelx, pContext) ) break;
			}

			if(pContext->m_nCurrentPos == nbegin) break;

			n ++;
			nbegin = pContext->m_nCurrentPos;
		}

		if(pContext->m_nCurrentPos == nbegin0)
		{
			while(n > n0 - 1)
			{
				while(ElxInterface_MatchNext(pData->m_base.m_pelx, pContext)) { }
				n --;
			}
		}
	}

	BufferData_Append(&pContext->m_stack, &nbegin00, 1);
	BufferData_Append(&pContext->m_stack, &n, 1);

	return 1;
}

int ElxGreedy_Match(PElxGreedy pData, PMatchContext pContext)
{
	if( ! ElxRepeat_Match((PElxRepeat)pData, pContext) )
		return 0;

	while( ! ElxGreedy_MatchVart(pData, pContext) )
	{
		if( ! ElxRepeat_MatchNext((PElxRepeat)pData, pContext) )
			return 0;
	}

	return 1;
}

int ElxGreedy_MatchNext(PElxGreedy pData, PMatchContext pContext)
{
	if( ElxGreedy_MatchNextVart(pData, pContext) )
		return 1;

	if( ! ElxRepeat_MatchNext((PElxRepeat)pData, pContext) )
		return 0;

	while( ! ElxGreedy_MatchVart(pData, pContext) )
	{
		if( ! ElxRepeat_MatchNext((PElxRepeat)pData, pContext) )
			return 0;
	}

	return 1;
}

PElxGreedy ElxGreedy_New(PElxInterface pElx, int nMin, int nMax)
{
	PElxGreedy pData = (PElxGreedy)malloc(sizeof(SElxGreedy));

	pData->m_base.m_base.m_fnMatch     = (VfnMatch)ElxGreedy_Match;
	pData->m_base.m_base.m_fnMatchNext = (VfnMatch)ElxGreedy_MatchNext;
	pData->m_base.m_base.m_fnRelease   = 0;

	pData->m_base.m_pelx   = pElx;
	pData->m_base.m_nfixed = nMin;

	pData->m_nvart = nMax - nMin;

	return pData;
}

int ElxIndependent_Match(PElxIndependent pData, PMatchContext pContext)
{
	int nbegin = pContext->m_nCurrentPos;
	int nsize  = pContext->m_stack.m_nSize;
	int ncsize = pContext->m_capturestack.m_nSize;

	int bsucc  = ElxInterface_Match(pData->m_pelx, pContext);

	BufferData_SetSize(&pContext->m_stack, nsize);

	if( bsucc )
	{
		BufferData_Append(&pContext->m_stack, &nbegin, 1);
		BufferData_Append(&pContext->m_stack, &ncsize, 1);
	}

	return bsucc;
}

int ElxIndependent_MatchNext(PElxIndependent pData, PMatchContext pContext)
{
	int ncsize;

	BufferData_Pop(&pContext->m_stack, &ncsize);
	BufferData_Pop(&pContext->m_stack, &pContext->m_nCurrentPos);

	BufferData_SetSize(&pContext->m_capturestack, ncsize);

	return 0;
}

PElxIndependent ElxIndependent_New(PElxInterface pElx)
{
	PElxIndependent pData = (PElxIndependent)malloc(sizeof(SElxIndependent));

	pData->m_base.m_fnMatch     = (VfnMatch)ElxIndependent_Match;
	pData->m_base.m_fnMatchNext = (VfnMatch)ElxIndependent_MatchNext;
	pData->m_base.m_fnRelease   = 0;

	pData->m_pelx = pElx;

	return pData;
}

int ElxList_Match(PElxList pData, PMatchContext pContext)
{
	int bol, stp, eol, n;

	if(pData->m_elxlist.m_nSize == 0)
		return 1;

	bol = pData->m_rightleft ? pData->m_elxlist.m_nSize : -1;
	stp = pData->m_rightleft ? -1 : 1;
	eol = pData->m_rightleft ? -1 : pData->m_elxlist.m_nSize;

	n = bol + stp;

	while(n != eol)
	{
		if(ElxInterface_Match(BufferData_GetPointerAt(&pData->m_elxlist, n), pContext))
		{
			n += stp;
		}
		else
		{
			n -= stp;

			while(n != bol && ! ElxInterface_MatchNext(BufferData_GetPointerAt(&pData->m_elxlist, n), pContext))
				n -= stp;

			if(n != bol)
				n += stp;
			else
				return 0;
		}
	}

	return 1;
}

int ElxList_MatchNext(PElxList pData, PMatchContext pContext)
{
	int bol, stp, eol, n;

	if(pData->m_elxlist.m_nSize == 0)
		return 0;

	bol = pData->m_rightleft ? pData->m_elxlist.m_nSize : -1;
	stp = pData->m_rightleft ? -1 : 1;
	eol = pData->m_rightleft ? -1 : pData->m_elxlist.m_nSize;

	n = eol - stp;

	while(n != bol && ! ElxInterface_MatchNext(BufferData_GetPointerAt(&pData->m_elxlist, n), pContext))
		n -= stp;

	if(n != bol)
		n += stp;
	else
		return 0;

	while(n != eol)
	{
		if(ElxInterface_Match(BufferData_GetPointerAt(&pData->m_elxlist, n), pContext))
		{
			n += stp;
		}
		else
		{
			n -= stp;

			while(n != bol && ! ElxInterface_MatchNext(BufferData_GetPointerAt(&pData->m_elxlist, n), pContext))
				n -= stp;

			if(n != bol)
				n += stp;
			else
				return 0;
		}
	}

	return 1;
}

void ElxList_Release(PElxList pData)
{
	BufferData_Release(&pData->m_elxlist);
}

PElxList ElxList_New(int bRightLeft)
{
	PElxList pData = (PElxList)malloc(sizeof(SElxList));

	pData->m_base.m_fnMatch     = (VfnMatch)ElxList_Match;
	pData->m_base.m_fnMatchNext = (VfnMatch)ElxList_MatchNext;
	pData->m_base.m_fnRelease   = (VfnElxRelease)ElxList_Release;

	BufferData_Init(&pData->m_elxlist, sizeof(PElxInterface));

	pData->m_rightleft = bRightLeft;

	return pData;
}

int ElxPossessive_Match(PElxPossessive pData, PMatchContext pContext)
{
	int nbegin = pContext->m_nCurrentPos;
	int nsize  = pContext->m_stack.m_nSize;
	int ncsize = pContext->m_capturestack.m_nSize;
	int bsucc  = 1;

	if( ! ElxRepeat_Match((PElxRepeat)pData, pContext) )
	{
		bsucc = 0;
	}
	else
	{
		while( ! ElxGreedy_MatchVart((PElxGreedy)pData, pContext) )
		{
			if( ! ElxRepeat_MatchNext((PElxRepeat)pData, pContext) )
			{
				bsucc = 0;
				break;
			}
		}
	}

	BufferData_SetSize(&pContext->m_stack, nsize);

	if( bsucc )
	{
		BufferData_Append(&pContext->m_stack, &nbegin, 1);
		BufferData_Append(&pContext->m_stack, &ncsize, 1);
	}

	return bsucc;
}

int ElxPossessive_MatchNext(PElxPossessive pData, PMatchContext pContext)
{
	int nbegin = 0, ncsize = 0;

	BufferData_Pop(&pContext->m_stack, &ncsize);
	BufferData_Pop(&pContext->m_stack, &nbegin);

	BufferData_SetSize(&pContext->m_capturestack, ncsize);
	pContext->m_nCurrentPos = nbegin;

	return 0;
}

PElxPossessive ElxPossessive_New(PElxInterface pElx, int nMin, int nMax)
{
	PElxPossessive pData = (PElxPossessive)malloc(sizeof(SElxPossessive));

	pData->m_base.m_base.m_base.m_fnMatch     = (VfnMatch)ElxPossessive_Match;
	pData->m_base.m_base.m_base.m_fnMatchNext = (VfnMatch)ElxPossessive_MatchNext;
	pData->m_base.m_base.m_base.m_fnRelease   = 0;

	pData->m_base.m_base.m_pelx   = pElx;
	pData->m_base.m_base.m_nfixed = nMin;

	pData->m_base.m_nvart = nMax - nMin;

	return pData;
}

int ElxRange_Contains(PElxRange pData, void * pch)
{
	int i;

	for(i=0; i<pData->m_ranges.m_nSize; i+=2)
	{
		if( (*pData->m_compare)(pch, BufferData_GetAt(&pData->m_ranges, i  )) >= 0 &&
			(*pData->m_compare)(pch, BufferData_GetAt(&pData->m_ranges, i+1)) <= 0 )
		{
			return 1;
		}
	}

	for(i=0; i<pData->m_embeds.m_nSize; i++)
	{
		if( ElxRange_Contains((PElxRange)BufferData_GetPointerAt(&pData->m_embeds, i), pch) )
		{
			return 1;
		}
	}

	for(i=0; i<pData->m_chars.m_nSize; i++)
	{
		if( (*pData->m_compare)(pch, BufferData_GetAt(&pData->m_chars, i)) == 0 )
		{
			return 1;
		}
	}

	return 0;
}

int ElxRange_Match(PElxRange pData, PMatchContext pContext)
{
	int at = pData->m_rightleft ? pContext->m_nCurrentPos - 1 : pContext->m_nCurrentPos;
	int bsucc;

	if(at < 0 || at >= pContext->m_string.m_nSize)
		return 0;

	bsucc = ElxRange_Contains(pData, BufferRefData_GetAt(&pContext->m_string, at, pData->m_chars.m_nSizeOfT));

	if( ! pData->m_byes )
		bsucc = ! bsucc;

	if( bsucc )
		pContext->m_nCurrentPos += pData->m_rightleft ? -1 : 1;

	return bsucc;
}

int ElxRange_MatchNext(PElxRange pData, PMatchContext pContext)
{
	pContext->m_nCurrentPos -= pData->m_rightleft ? -1 : 1;
	return 0;
}

void ElxRange_Release(PElxRange pData)
{
	BufferData_Release(&pData->m_chars);
	BufferData_Release(&pData->m_ranges);
	BufferData_Release(&pData->m_embeds);
}

int ElxRange_Compare1(void * pch1, void * pch2)
{
	if(*(unsigned char*)pch1 < *(unsigned char*)pch2)
		return -1;
	else if(*(unsigned char*)pch1 == *(unsigned char*)pch2)
		return 0;
	else
		return 1;
}

int ElxRange_Compare2(void * pch1, void * pch2)
{
	if(*(unsigned short*)pch1 < *(unsigned short*)pch2)
		return -1;
	else if(*(unsigned short*)pch1 == *(unsigned short*)pch2)
		return 0;
	else
		return 1;
}

int ElxRange_Compare4(void * pch1, void * pch2)
{
	if(*(unsigned long*)pch1 < *(unsigned long*)pch2)
		return -1;
	else if(*(unsigned long*)pch1 == *(unsigned long*)pch2)
		return 0;
	else
		return 1;
}

PElxRange ElxRange_New(int nSizeOfT, int bRightLeft, int bYes)
{
	PElxRange pData = (PElxRange)malloc(sizeof(SElxRange));

	pData->m_base.m_fnMatch     = (VfnMatch)ElxRange_Match;
	pData->m_base.m_fnMatchNext = (VfnMatch)ElxRange_MatchNext;
	pData->m_base.m_fnRelease   = (VfnElxRelease)ElxRange_Release;

	BufferData_Init(&pData->m_chars , nSizeOfT);
	BufferData_Init(&pData->m_ranges, nSizeOfT);
	BufferData_Init(&pData->m_embeds, sizeof(PElxInterface));

	pData->m_rightleft = bRightLeft;
	pData->m_byes      = bYes;

	switch(nSizeOfT)
	{
	case 1: pData->m_compare = ElxRange_Compare1; break;
	case 2: pData->m_compare = ElxRange_Compare2; break;
	case 4: pData->m_compare = ElxRange_Compare4; break;
	default:pData->m_compare = 0;
	}

	return pData;
}

int ElxRecursive_Match(PElxRecursive pData, PMatchContext pContext)
{
	return pData->m_pelx ? ElxInterface_Match(pData->m_pelx, pContext) : 1;
}

int ElxRecursive_MatchNext(PElxRecursive pData, PMatchContext pContext)
{
	return pData->m_pelx ? ElxInterface_MatchNext(pData->m_pelx, pContext) : 0;
}

void ElxRecursive_Release(PElxRecursive pData)
{
	BufferData_Release(&pData->m_name);
}

PElxRecursive ElxRecursive_New(int nNumber, int nSizeOfT)
{
	PElxRecursive pData = (PElxRecursive)malloc(sizeof(SElxRecursive));

	pData->m_base.m_fnMatch     = (VfnMatch)ElxRecursive_Match;
	pData->m_base.m_fnMatchNext = (VfnMatch)ElxRecursive_MatchNext;
	pData->m_base.m_fnRelease   = (VfnElxRelease)ElxRecursive_Release;

	pData->m_pelx   = 0;
	pData->m_number = nNumber;

	BufferData_Init(&pData->m_name, nSizeOfT);

	return pData;
}

int ElxReluctant_MatchVart(PElxReluctant pData, PMatchContext pContext)
{
	int n = 0;
	BufferData_Append(&pContext->m_stack, &n, 1);

	return 1;
}

int ElxReluctant_MatchNextVart(PElxReluctant pData, PMatchContext pContext)
{
	int n = 0, nbegin = pContext->m_nCurrentPos;

	BufferData_Pop(&pContext->m_stack, &n);

	if(n < pData->m_nvart && ElxInterface_Match(pData->m_base.m_pelx, pContext))
	{
		while(pContext->m_nCurrentPos == nbegin)
		{
			if( ! ElxInterface_MatchNext(pData->m_base.m_pelx, pContext) ) break;
		}

		if(pContext->m_nCurrentPos != nbegin)
		{
			n ++;

			BufferData_Append(&pContext->m_stack, &nbegin, 1);
			BufferData_Append(&pContext->m_stack, &n, 1);

			return 1;
		}
	}

	while(n > 0)
	{
		BufferData_Pop(&pContext->m_stack, &nbegin);

		while( ElxInterface_MatchNext(pData->m_base.m_pelx, pContext) )
		{
			if(pContext->m_nCurrentPos != nbegin)
			{
				BufferData_Append(&pContext->m_stack, &nbegin, 1);
				BufferData_Append(&pContext->m_stack, &n, 1);

				return 1;
			}
		}

		n --;
	}

	return 0;
}

int ElxReluctant_Match(PElxReluctant pData, PMatchContext pContext)
{
	if( ! ElxRepeat_Match((PElxRepeat)pData, pContext) )
		return 0;

	while( ! ElxReluctant_MatchVart(pData, pContext) )
	{
		if( ! ElxRepeat_MatchNext((PElxRepeat)pData, pContext) )
			return 0;
	}

	return 1;
}

int ElxReluctant_MatchNext(PElxReluctant pData, PMatchContext pContext)
{
	if( ElxReluctant_MatchNextVart(pData, pContext) )
		return 1;

	if( ! ElxRepeat_MatchNext((PElxRepeat)pData, pContext) )
		return 0;

	while( ! ElxReluctant_MatchVart(pData, pContext) )
	{
		if( ! ElxRepeat_MatchNext((PElxRepeat)pData, pContext) )
			return 0;
	}

	return 1;
}

PElxReluctant ElxReluctant_New(PElxInterface pElx, int nMin, int nMax)
{
	PElxReluctant pData = (PElxReluctant)malloc(sizeof(SElxReluctant));

	pData->m_base.m_base.m_fnMatch     = (VfnMatch)ElxReluctant_Match;
	pData->m_base.m_base.m_fnMatchNext = (VfnMatch)ElxReluctant_MatchNext;
	pData->m_base.m_base.m_fnRelease   = 0;

	pData->m_base.m_pelx   = pElx;
	pData->m_base.m_nfixed = nMin;

	pData->m_nvart = nMax - nMin;

	return pData;
}

int ElxString_Match(PElxString pData, PMatchContext pContext)
{
	SBufferRefData ref;
	int bsucc;

	ref.m_nSize = pData->m_string.m_nSize;

	if(pData->m_rightleft)
	{
		if(pContext->m_nCurrentPos < ref.m_nSize)
			return 0;

		ref.m_pT = BufferRefData_GetAt(&pContext->m_string, pContext->m_nCurrentPos - ref.m_nSize, pData->m_string.m_nSizeOfT);
		bsucc = ! (*pData->m_compare)(&pData->m_string, &ref);

		if( bsucc ) pContext->m_nCurrentPos -= ref.m_nSize;
	}
	else
	{
		if(pContext->m_nCurrentPos + ref.m_nSize > pContext->m_string.m_nSize)
			return 0;

		ref.m_pT = BufferRefData_GetAt(&pContext->m_string, pContext->m_nCurrentPos, pData->m_string.m_nSizeOfT);
		bsucc = ! (*pData->m_compare)(&pData->m_string, &ref);

		if( bsucc ) pContext->m_nCurrentPos += ref.m_nSize;
	}

	return bsucc;
}

int ElxString_MatchNext(PElxString pData, PMatchContext pContext)
{
	if(pData->m_rightleft)
		pContext->m_nCurrentPos += pData->m_string.m_nSize;
	else
		pContext->m_nCurrentPos -= pData->m_string.m_nSize;

	return 0;
}

void ElxString_Release(PElxString pData)
{
	BufferData_Release(&pData->m_string);
}

PElxString ElxString_New(void * szFixed, int nLen, int nSizeOfT, int bRightLeft, int bIgnoreCase)
{
	PElxString pData = (PElxString)malloc(sizeof(SElxString));

	pData->m_base.m_fnMatch     = (VfnMatch)ElxString_Match;
	pData->m_base.m_fnMatchNext = (VfnMatch)ElxString_MatchNext;
	pData->m_base.m_fnRelease   = (VfnElxRelease)ElxString_Release;

	FixedBufData_Init(&pData->m_string, nSizeOfT);
	BufferData_Append(&pData->m_string, szFixed, nLen);

	pData->m_rightleft = bRightLeft;

	switch(nSizeOfT)
	{
	case 1: pData->m_compare = bIgnoreCase ? StringRefData_Compare1I : StringRefData_Compare1; break;
	case 2: pData->m_compare = bIgnoreCase ? StringRefData_Compare2I : StringRefData_Compare2; break;
	case 4: pData->m_compare = bIgnoreCase ? StringRefData_Compare4I : StringRefData_Compare4; break;
	default:pData->m_compare = 0;
	}

	return pData;
}

#include <limits.h>

#define max(a,b)    (((a) > (b)) ? (a) : (b))
#define min(a,b)    (((a) < (b)) ? (a) : (b))

void BuildContext_Reset(PBuildContext pData)
{
	int i;
	for(i=0; i<pData->m_objlist.m_nSize; i++)
	{
		PElxInterface pElx = (PElxInterface)BufferData_GetPointerAt(&pData->m_objlist, i);
		ElxInterface_Delete(pElx);
	}

	pData->m_nnextpos      = 0;
	pData->m_ncharsetdepth = 0;
	pData->m_nmaxnumber    = 0;
	pData->m_nnextnamed    = 0;
	pData->m_bquoted       = 0;
	pData->m_quote_fun     = 0;

	BufferData_SetSize(&pData->m_objlist           , 0);
	BufferData_SetSize(&pData->m_grouplist         , 0);
	BufferData_SetSize(&pData->m_recursivelist     , 0);
	BufferData_SetSize(&pData->m_namedlist         , 0);
	BufferData_SetSize(&pData->m_namedbackreflist  , 0);
	BufferData_SetSize(&pData->m_namedconditionlist, 0);
}

void BuildContext_Release(PBuildContext pData)
{
	BuildContext_Reset(pData);

	BufferData_Release(&pData->m_objlist           );
	BufferData_Release(&pData->m_grouplist         );
	BufferData_Release(&pData->m_recursivelist     );
	BufferData_Release(&pData->m_namedlist         );
	BufferData_Release(&pData->m_namedbackreflist  );
	BufferData_Release(&pData->m_namedconditionlist);
}

void BuildContext_Delete(PBuildContext pData)
{
	BuildContext_Release(pData);
	free(pData);
}

int BuildContext_ReadHex(PBuildContext pData, void * pStr, int nLen, long * val)
{
	SBufferRefData ref;
	int i;

	ref.m_pT    = pStr;
	ref.m_nSize = nLen;
	*val = 0;

	for(i=0; i<nLen; i++)
	{
		long ch = (*pData->m_getch)(&ref, i);
		if(ch >= '0' && ch <= '9')
			*val = *val * 16 + (ch - '0');
		else if(ch >= 'A' && ch <= 'F')
			*val = *val * 16 + (ch - 'A' + 10);
		else if(ch >= 'a' && ch <= 'f')
			*val = *val * 16 + (ch - 'a' + 10);
		else
			return i;
	}

	return i;
}

int BuildContext_ReadDec(PBuildContext pData, void * pStr, int nLen, long * val)
{
	SBufferRefData ref;
	int i = 0, j;

	ref.m_pT    = pStr;
	ref.m_nSize = nLen;
	*val = 0;

	for(; i<nLen; i++)
	{
		long ch = (*pData->m_getch)(&ref, i);
		if(ch != ' ' && ch != '\t')
			break;
	}

	for(j=i; i<nLen; i++)
	{
		long ch = (*pData->m_getch)(&ref, i);
		if(ch >= '0' && ch <= '9')
			*val = *val * 10 + (ch - '0');
		else
			break;
	}

	if(j == i)
		return 0;

	for(; i<nLen; i++)
	{
		long ch = (*pData->m_getch)(&ref, i);
		if(ch != ' ' && ch != '\t')
			break;
	}

	return i;
}

int BuildContext_GetNamedNumber(PBuildContext pData, PBufferData pName)
{
	int i;

	for(i=0; i<pData->m_namedlist.m_nSize; i++)
	{
		PElxList    plist = (PElxList   )BufferData_GetPointerAt(&pData->m_namedlist, i);
		PElxBracket pleft = (PElxBracket)BufferData_GetPointerAt(&plist->m_elxlist, 0);

		if((*pData->m_compare)(&pleft->m_name, pName) == 0)
		{
			return pleft->m_number;
		}
	}

	return -3;
}

#define IsCurrChar(pData, c, t) (pData->curr.ch == (c) && pData->curr.type == (t))
#define IsNextChar(pData, c, t) (pData->next.ch == (c) && pData->next.type == (t))
#define IsNex2Char(pData, c, t) (pData->nex2.ch == (c) && pData->nex2.type == (t))

void BuildContext_MoveNext(PBuildContext pData)
{
	pData->curr = pData->next;
	pData->next = pData->nex2;

	while(1)
	{
		int  delta;
		long ch, ch1, ch2;

		/* end of pattern */
		if(pData->m_nnextpos >= pData->m_pattern.m_nSize)
		{
			pData->nex2.ch   = 0;
			pData->nex2.type = 1;
			return;
		}

		delta = 1;
		ch    = (*pData->m_getch)(&pData->m_pattern, pData->m_nnextpos);
		ch1   = (pData->m_nnextpos + 1) < pData->m_pattern.m_nSize ? (*pData->m_getch)(&pData->m_pattern, pData->m_nnextpos + 1) : 0;

		/* quoted */
		if(pData->m_bquoted)
		{
			if(ch == '\\' && ch1 == 'E')
			{
				pData->m_quote_fun = 0;
				pData->m_bquoted   = 0;
				pData->m_nnextpos += 2;
				continue;
			}

			if(pData->m_quote_fun != 0)
			{
				pData->nex2.ch   = (*pData->m_quote_fun)((int)ch);
				pData->nex2.type = 0;
			}
			else
			{
				pData->nex2.ch   = ch;
				pData->nex2.type = 0;
			}

			pData->m_nnextpos += delta;
			return;
		}

		/* parse */
		switch(ch)
		{
		case '\\':
			{
				/* back ref */
				if(ch1 >= '0' && ch1 <= '9')
				{
					pData->nex2.ch   = ch;
					pData->nex2.type = 1;
					break;
				}

				/* escaped */
				delta = 2;
				switch(ch1)
				{
				case 'A':
				case 'Z':
				case 'z':
				case 'w':
				case 'W':
				case 's':
				case 'S':
				case 'B':
				case 'd':
				case 'D':
				case 'k':
				case 'g':
					pData->nex2.ch   = ch1;
					pData->nex2.type = 1;
					break;

				case 'b':
					if(pData->m_ncharsetdepth > 0)
					{
						pData->nex2.ch   = '\b';
						pData->nex2.type = 0;
					}
					else
					{
						pData->nex2.ch   = ch1;
						pData->nex2.type = 1;
					}
					break;

				case 'x':
					ch2   = (pData->m_nnextpos + 2) < pData->m_pattern.m_nSize ? (*pData->m_getch)(&pData->m_pattern, pData->m_nnextpos + 2) : 0;
					if(ch2 != '{')
					{
						long hex;
						int  red = BuildContext_ReadHex(pData, BufferRefData_GetAt(&pData->m_pattern, pData->m_nnextpos + 2, pData->m_nSizeOfT), 2, &hex);

						if(red > 0)
						{
							pData->nex2.ch = hex;
							pData->nex2.type = 0;
						}
						else
						{
							pData->nex2.ch = ch1;
							pData->nex2.type = 0;
						}

						delta += red;
						break;
					}

				case 'u':
					ch2   = (pData->m_nnextpos + 2) < pData->m_pattern.m_nSize ? (*pData->m_getch)(&pData->m_pattern, pData->m_nnextpos + 2) : 0;
					if(ch2 != '{')
					{
						long hex;
						int  red = BuildContext_ReadHex(pData, BufferRefData_GetAt(&pData->m_pattern, pData->m_nnextpos + 2, pData->m_nSizeOfT), 4, &hex);

						if(red > 0)
						{
							pData->nex2.ch = hex;
							pData->nex2.type = 0;
						}
						else
						{
							pData->nex2.ch = ch1;
							pData->nex2.type = 0;
						}

						delta += red;
					}
					else
					{
						long hex;
						int  red = BuildContext_ReadHex(pData, BufferRefData_GetAt(&pData->m_pattern, pData->m_nnextpos + 3, pData->m_nSizeOfT), 8, &hex);

						delta += red + 1;

						while(pData->m_nnextpos + delta < pData->m_pattern.m_nSize && (*pData->m_getch)(&pData->m_pattern, pData->m_nnextpos + delta) != '}')
							delta ++;

						delta ++; /* } */

						pData->nex2.ch   = hex;
						pData->nex2.type = 0;
					}
					break;

				case 'a':
					pData->nex2.ch   = '\a';
					pData->nex2.type = 0;
					break;

				case 'f':
					pData->nex2.ch   = '\f';
					pData->nex2.type = 0;
					break;

				case 'n':
					pData->nex2.ch   = '\n';
					pData->nex2.type = 0;
					break;

				case 'r':
					pData->nex2.ch   = '\r';
					pData->nex2.type = 0;
					break;

				case 't':
					pData->nex2.ch   = '\t';
					pData->nex2.type = 0;
					break;

				case 'v':
					pData->nex2.ch   = '\v';
					pData->nex2.type = 0;
					break;

				case 'e':
					pData->nex2.ch   = 27;
					pData->nex2.type = 0;
					break;

				case 'G':
					if(pData->m_ncharsetdepth > 0)
					{
						pData->m_nnextpos += 2;
						continue;
					}
					else
					{
						pData->nex2.ch   = ch1;
						pData->nex2.type = 1;
						break;
					}

				case 'L':
					if( ! pData->m_quote_fun ) pData->m_quote_fun = tolower;

				case 'U':
					if( ! pData->m_quote_fun ) pData->m_quote_fun = toupper;

				case 'Q':
					pData->m_bquoted   = 1;
					pData->m_nnextpos += 2;
					continue;

				case 'E':
					pData->m_bquoted   = 0;
					pData->m_quote_fun = 0;
					pData->m_nnextpos += 2;
					continue;

				case 0:
					pData->nex2.ch   = pData->m_nnextpos + 1 >= pData->m_pattern.m_nSize ? ch : ch1;
					pData->nex2.type = 0;
					break;

				default:
					pData->nex2.ch   = ch1;
					pData->nex2.type = 0;
					break;
				}
			}
			break;

		case '*':
		case '+':
		case '?':
		case '.':
		case '{':
		case '}':
		case ')':
		case '|':
		case '$':
			pData->nex2.ch   = ch;
			pData->nex2.type = pData->m_ncharsetdepth > 0 ? 0 : 1;
			break;

		case '-':
			pData->nex2.ch   = ch;
			pData->nex2.type = pData->m_ncharsetdepth > 0 ? 1 : 0;
			break;

		case '(':
			ch2   = (pData->m_nnextpos + 2) < pData->m_pattern.m_nSize ? (*pData->m_getch)(&pData->m_pattern, pData->m_nnextpos + 2) : 0;
			/* remark */
			if(ch1 == '?' && ch2 == '#')
			{
				pData->m_nnextpos += 2;

				while(pData->m_nnextpos < pData->m_pattern.m_nSize)
				{
					if((*pData->m_getch)(&pData->m_pattern, pData->m_nnextpos) == ')')
						break;

					pData->m_nnextpos ++;
				}

				if((*pData->m_getch)(&pData->m_pattern, pData->m_nnextpos) == ')')
				{
					pData->m_nnextpos ++;
					continue;
				}
			}
			else
			{
				pData->nex2.ch   = ch;
				pData->nex2.type = pData->m_ncharsetdepth > 0 ? 0 : 1;
			}
			break;

		case '#':
			if(pData->m_nflags & 0x20) /* extended */
			{
				pData->m_nnextpos ++;
				while(pData->m_nnextpos < pData->m_pattern.m_nSize)
				{
					if((*pData->m_getch)(&pData->m_pattern, pData->m_nnextpos) == '\n' || (*pData->m_getch)(&pData->m_pattern, pData->m_nnextpos) == '\r')
						break;

					pData->m_nnextpos ++;
				}
				continue;
			}
			else
			{
				pData->nex2.ch   = ch;
				pData->nex2.type = 0;
				break;
			}

		case ' ':
		case '\f':
		case '\n':
		case '\r':
		case '\t':
		case '\v':
			if(pData->m_nflags & 0x20) /* extended */
			{
				pData->m_nnextpos ++;
				continue;
			}
			else
			{
				pData->nex2.ch   = ch;
				pData->nex2.type = 0;
				break;
			}

		case '[':
			if(pData->m_ncharsetdepth == 0 || ch1 == ':')
			{
				pData->m_ncharsetdepth ++;
				pData->nex2.ch   = ch;
				pData->nex2.type = 1;
			}
			else
			{
				pData->nex2.ch   = ch;
				pData->nex2.type = 0;
			}
			break;

		case ']':
			if(pData->m_ncharsetdepth > 0)
			{
				pData->m_ncharsetdepth --;
				pData->nex2.ch   = ch;
				pData->nex2.type = 1;
			}
			else
			{
				pData->nex2.ch   = ch;
				pData->nex2.type = 0;
			}
			break;

		case ':':
			pData->nex2.ch   = ch;
			pData->nex2.type = IsNextChar(pData, '[', 1) ? 1 : 0;
			break;

		case '^':
			pData->nex2.ch   = ch;
			pData->nex2.type = pData->m_ncharsetdepth == 0 || IsNextChar(pData, '[', 1) || (IsCurrChar(pData, '[', 1) && IsNextChar(pData, ':', 1)) ? 1 : 0;
			break;

		case 0:
			pData->nex2.ch   = ch;
			pData->nex2.type = pData->m_nnextpos >= pData->m_pattern.m_nSize ? 1 : 0;
			break;

		default:
			pData->nex2.ch   = ch;
			pData->nex2.type = 0;
		}

		/* delta */
		pData->m_nnextpos += delta;
		return;
	}
}

PElxInterface BuildContext_Keep(PBuildContext pData, void * pElx)
{
	BufferData_Append(&pData->m_objlist, &pElx, 1);
	return (PElxInterface)pElx;
}

PElxInterface BuildContext_BuildAlternative(PBuildContext pData, int nFlags);
PElxInterface BuildContext_BuildList(PBuildContext pData, int * pFlags);

PElxInterface BuildContext_BuildRecursive(PBuildContext pData, int * pFlags)
{
	BuildContext_MoveNext(pData);

	if( ! IsCurrChar(pData, '?', 1) )
	{
		int nThisBracket = ++ pData->m_nmaxnumber;

		PElxList      pList     = (PElxList)BuildContext_Keep(pData, ElxList_New(*pFlags & 0x10));
		PElxInterface pBracket1 = BuildContext_Keep(pData, ElxBracket_New(nThisBracket, *pFlags & 0x10 ? 1 : 0, pData->m_nSizeOfT));
		PElxInterface pAlter    = BuildContext_BuildAlternative(pData, *pFlags);
		PElxInterface pBracket2 = BuildContext_Keep(pData, ElxBracket_New(nThisBracket, *pFlags & 0x10 ? 0 : 1, pData->m_nSizeOfT));

		BufferData_Append(&pList->m_elxlist, &pBracket1, 1);
		BufferData_Append(&pList->m_elxlist, &pAlter   , 1);
		BufferData_Append(&pList->m_elxlist, &pBracket2, 1);

		BufferData_Prepare(&pData->m_grouplist, nThisBracket, 0);
		*(PElxInterface*)BufferData_GetAt(&pData->m_grouplist, nThisBracket) = (PElxInterface)pList;

		BuildContext_MoveNext(pData);

		return (PElxInterface)pList;
	}
	else
	{
		PElxInterface pElx = 0;
		int  bNegative = 0;
		long named_end;

		BuildContext_MoveNext(pData);

		switch(pData->curr.ch)
		{
		case '!':
			bNegative = 1;

		case '=':
			BuildContext_MoveNext(pData);
			pElx = BuildContext_Keep(pData, ElxAssert_New(BuildContext_BuildAlternative(pData, *pFlags & ~0x10), bNegative));
			break;

		case '<':
			switch(pData->next.ch)
			{
			case '!':
				bNegative = 1;

			case '=':
				BuildContext_MoveNext(pData);
				BuildContext_MoveNext(pData);
				pElx = BuildContext_Keep(pData, ElxAssert_New(BuildContext_BuildAlternative(pData, *pFlags | 0x10), bNegative));
				break;
			}
			if(pElx != 0) break;

		case 'P':
			if(pData->curr.ch == 'P') BuildContext_MoveNext(pData);

		case '\'':
			{
				PElxList      pList     = (PElxList   )BuildContext_Keep(pData, ElxList_New(*pFlags & 0x10));
				PElxBracket   pBracket1 = (PElxBracket)BuildContext_Keep(pData, ElxBracket_New(-1, *pFlags & 0x10 ? 1 : 0, pData->m_nSizeOfT));
				PElxInterface pAlter;
				PElxBracket   pBracket2 = (PElxBracket)BuildContext_Keep(pData, ElxBracket_New(-1, *pFlags & 0x10 ? 0 : 1, pData->m_nSizeOfT));

				PBufferData name = &pBracket1->m_name;
				int  pos;
				long num;

				if     (pData->curr.ch == '<' ) named_end = '>';
				else if(pData->curr.ch == '\'') named_end = '\'';
				BuildContext_MoveNext(pData);

				while( ! IsCurrChar(pData, 0, 1) && pData->curr.ch != named_end )
				{
					BufferData_Append(name, (*pData->m_ltochar)(&pData->curr.ch), 1);
					BuildContext_MoveNext(pData);
				}
				BuildContext_MoveNext(pData);

				if((pos = BuildContext_ReadDec(pData, name->m_pT, name->m_nSize, &num)) > 0 && pos == name->m_nSize)
				{
					pBracket1->m_number = (int)num;
					pBracket2->m_number = (int)num;

					BufferData_Release(name);
				}

				pAlter = BuildContext_BuildAlternative(pData, *pFlags);

				BufferData_Append(&pList->m_elxlist, &pBracket1, 1);
				BufferData_Append(&pList->m_elxlist, &pAlter   , 1);
				BufferData_Append(&pList->m_elxlist, &pBracket2, 1);

				BufferData_Append(&pData->m_namedlist, &pList, 1);

				pElx = (PElxInterface)pList;
			}
			break;

		case '>':
			BuildContext_MoveNext(pData);
			pElx = BuildContext_Keep(pData, ElxIndependent_New(BuildContext_BuildAlternative(pData, * pFlags)));
			break;

		case 'R':
			{
				PElxRecursive pRecursive = (PElxRecursive)BuildContext_Keep(pData, ElxRecursive_New(-3, pData->m_nSizeOfT));
				PBufferData   name       = &pRecursive->m_name;
				int  pos;
				long num;

				BuildContext_MoveNext(pData);

				if     (pData->curr.ch == '<') named_end = '>';
				else if(pData->curr.ch = '\'') named_end = '\'';
				else                           named_end = ')';

				if(named_end != ')') BuildContext_MoveNext(pData);

				while( ! IsCurrChar(pData, 0, 1) && pData->curr.ch != named_end )
				{
					BufferData_Append(name, (*pData->m_ltochar)(&pData->curr.ch), 1);
					BuildContext_MoveNext(pData);
				}
				while( ! IsCurrChar(pData, 0, 1) && pData->curr.ch != ')' ) BuildContext_MoveNext(pData);

				if((pos = BuildContext_ReadDec(pData, name->m_pT, name->m_nSize, &num)) > 0 && pos == name->m_nSize)
				{
					pRecursive->m_number = (int)num;
					BufferData_Release(name);
				}

				BufferData_Append(&pData->m_recursivelist, &pRecursive, 1);

				pElx = (PElxInterface)pRecursive;
			}
			break;

		case '(':
			{
				PElxConditional pConditional = (PElxConditional)BuildContext_Keep(pData, ElxConditional_New(pData->m_nSizeOfT));
				int flags = *pFlags;

				if(IsNextChar(pData, '?', 1))
				{
					pConditional->m_pelxyes = BuildContext_BuildRecursive(pData, pFlags);
				}
				else
				{
					/* save */
					SCharInfo curr = pData->curr;
					SCharInfo next = pData->next;
					SCharInfo nex2 = pData->nex2;
					int nnextpos   = pData->m_nnextpos;

					PBufferData name = &pConditional->m_name;
					int  pos;
					long num;

					BuildContext_MoveNext(pData);

					while( ! IsCurrChar(pData, 0, 1) && ! IsCurrChar(pData, ')', 1) )
					{
						BufferData_Append(name, (*pData->m_ltochar)(&pData->curr.ch), 1);
						BuildContext_MoveNext(pData);
					}
					BuildContext_MoveNext(pData);

					if((pos = BuildContext_ReadDec(pData, name->m_pT, name->m_nSize, &num)) > 0 && pos == name->m_nSize)
					{
						pConditional->m_number = (int)num;
						BufferData_Release(name);
					}
					else
					{
						/* restore */
						pData->curr = curr;
						pData->next = next;
						pData->nex2 = nex2;
						pData->m_nnextpos = nnextpos;

						BuildContext_MoveNext(pData);

						pConditional->m_pelxask = BuildContext_Keep(pData, ElxAssert_New(BuildContext_BuildAlternative(pData, *pFlags), 1));
						BuildContext_MoveNext(pData);

						pConditional->m_number = -1;
						BufferData_Append(&pData->m_namedconditionlist, &pConditional, 1);
					}
				}

				pConditional->m_pelxyes = BuildContext_BuildList(pData, &flags);

				if(IsCurrChar(pData, '|', 1))
				{
					BuildContext_MoveNext(pData);
					pConditional->m_pelxno = BuildContext_BuildAlternative(pData, flags);
				}

				pElx = (PElxInterface)pConditional;
			}
			break;

		default:
			if(pData->curr.ch >= '0' && pData->curr.ch <= '9')
			{
				SBufferData rto;
				PElxRecursive pRecursive;
				long num;

				BufferData_Init(&rto, pData->m_nSizeOfT);

				while( ! IsCurrChar(pData, 0, 1) && ! IsCurrChar(pData, ')', 1) )
				{
					BufferData_Append(&rto, (*pData->m_ltochar)(&pData->curr.ch), 1);
					BuildContext_MoveNext(pData);
				}

				BuildContext_ReadDec(pData, rto.m_pT, rto.m_nSize, &num);
				BufferData_Release(&rto);

				pRecursive = (PElxRecursive)BuildContext_Keep(pData, ElxRecursive_New((int)num, pData->m_nSizeOfT));
				BufferData_Append(&pData->m_recursivelist, &pRecursive, 1);

				pElx = (PElxInterface)pRecursive;
			}
			else
			{
				int flags = *pFlags;

				while( ! IsCurrChar(pData, 0, 1) && ! IsCurrChar(pData, ':', 0) && ! IsCurrChar(pData, '(', 1) && ! IsCurrChar(pData, ')', 1) )
				{
					int tochange = 0;

					switch(pData->curr.ch)
					{
					case 'i':
					case 'I':
						tochange = 0x08;
						break;

					case 's':
					case 'S':
						tochange = 0x01;
						break;

					case 'm':
					case 'M':
						tochange = 0x02;
						break;

					case 'g':
					case 'G':
						tochange = 0x04;
						break;

					case '-':
						bNegative = 1;
						break;
					}

					if(bNegative)
						flags &= ~tochange;
					else
						flags |=  tochange;

					BuildContext_MoveNext(pData);
				}

				if(IsCurrChar(pData, ':', 0) || IsCurrChar(pData, '(', 1))
				{
					if(pData->curr.ch == ':') BuildContext_MoveNext(pData);

					pElx = BuildContext_BuildAlternative(pData, flags);
				}
				else
				{
					*pFlags = flags;

					pElx = BuildContext_Keep(pData, ElxEmpty_New());
				}
			}
			break;
		}

		BuildContext_MoveNext(pData);

		return pElx;
	}
}

#define AppendChar( pRange, c     ) do { ch1 = c; BufferData_Append(&pRange->m_chars, (pData->m_ltochar)(&ch1), 1); } while(0)
#define AppendRange(pRange, c1, c2) do { ch1 = c1; ch2 = c2; BufferData_Append(&pRange->m_ranges, (pData->m_ltochar)(&ch1), 1); BufferData_Append(&pRange->m_ranges, (pData->m_ltochar)(&ch2), 1); } while(0)

PElxInterface BuildContext_BuildCharset(PBuildContext pData, int * pFlags)
{
	long ch = pData->curr.ch, ch1, ch2;
	PElxRange pRange = (PElxRange)BuildContext_Keep(pData, ElxRange_New(pData->m_nSizeOfT, *pFlags & 0x10, 1));

	BuildContext_MoveNext(pData);

	switch(ch)
	{
	case '.':
		pRange->m_byes = 0;
		if((*pFlags & 0x01) == 0) AppendChar(pRange, '\n');
		break;

	case 'W':
		pRange->m_byes = 0;

	case 'w':
		AppendRange(pRange, 'A', 'Z');
		AppendRange(pRange, 'a', 'z');
		AppendRange(pRange, '0', '9');
		AppendChar (pRange, '_');
		break;

	case 'S':
		pRange->m_byes = 0;

	case 's':
		AppendChar(pRange, '_');
		AppendChar(pRange, '\t');
		AppendChar(pRange, '\r');
		AppendChar(pRange, '\n');
		break;

	case 'D':
		pRange->m_byes = 0;

	case 'd':
		AppendRange(pRange, '0', '9');
		break;

	case '[':
		if(IsCurrChar(pData, ':', 1))
		{
			BuildContext_MoveNext(pData);
			while( ! IsCurrChar(pData, 0, 1) && ! IsCurrChar(pData, ']', 1) )
			{
				BuildContext_MoveNext(pData);
			}
			BuildContext_MoveNext(pData);
			
			return (PElxInterface)pRange;
		}
		
		if(IsCurrChar(pData, '^', 1))
		{
			pRange->m_byes = 0;
			BuildContext_MoveNext(pData);
		}

		while( ! IsCurrChar(pData, 0, 1) && ! IsCurrChar(pData, ']', 1) )
		{
			ch1 = pData->curr.ch;

			if(pData->curr.type == 1 && (
					ch1 == '.' || ch1 == 'w' || ch1 == 'W' || ch1 == 's' || ch1 == 'S' || ch1 == 'd' || ch1 == 'D' ||
					(ch1 == '[' && IsNextChar(pData, ':', 1))
			))
			{
				PElxInterface pElx = BuildContext_BuildCharset(pData, pFlags);
				BufferData_Append(&pRange->m_embeds, &pElx, 1);
			}
			else if(IsNextChar(pData, '-', 1) && pData->nex2.type == 0)
			{
				AppendRange(pRange, pData->curr.ch, pData->nex2.ch);

				BuildContext_MoveNext(pData);
				BuildContext_MoveNext(pData);
				BuildContext_MoveNext(pData);
			}
			else
			{
				AppendChar(pRange, pData->curr.ch);

				BuildContext_MoveNext(pData);
			}
		}

		BuildContext_MoveNext(pData);

		break;
	}

	return (PElxInterface)pRange;
}

PElxInterface BuildContext_BuildBoundary(PBuildContext pData, int * pFlags)
{
	long ch = pData->curr.ch;
	PElxBoundary pBoundary = (PElxBoundary)ElxBoundary_New(pData->m_nSizeOfT, 0, 1);

	BuildContext_MoveNext(pData);

	switch(ch)
	{
	case '^':
		pBoundary->m_type = *pFlags & 0x02 ? deelx_line_begin : deelx_file_begin;
		break;

	case '$':
		pBoundary->m_type = *pFlags & 0x02 ? deelx_line_end : deelx_file_end;
		break;

	case 'B':
		pBoundary->m_byes = 0;

	case 'b':
		pBoundary->m_type = deelx_word_edge;
		break;

	case 'A':
		pBoundary->m_type = deelx_file_begin;
		break;

	case 'Z':
		pBoundary->m_type = deelx_file_end_n;
		break;

	case 'z':
		pBoundary->m_type = deelx_file_end;
		break;

	case 'G':
		pBoundary->m_type = deelx_this_begin;
		break;
	}

	return (PElxInterface)pBoundary;
}

PElxInterface BuildContext_BuildBackref(PBuildContext pData, int * pFlags)
{
	BuildContext_MoveNext(pData);

	if(pData->curr.ch == '<' || pData->curr.ch == '\'')
	{
		PElxBackRefer pBackref = (PElxBackRefer)BuildContext_Keep(pData, ElxBackRefer_New(-1, pData->m_nSizeOfT, *pFlags & 0x10, *pFlags & 0x08));

		PBufferData name = &pBackref->m_name;
		int  pos;
		long num;
		long named_end = pData->curr.ch == '<' ? '>' : '\'';

		BuildContext_MoveNext(pData);

		while( ! IsCurrChar(pData, 0, 1) && pData->curr.ch != named_end )
		{
			BufferData_Append(name, (*pData->m_ltochar)(&pData->curr.ch), 1);
			BuildContext_MoveNext(pData);
		}
		BuildContext_MoveNext(pData);

		if((pos = BuildContext_ReadDec(pData, name->m_pT, name->m_nSize, &num)) > 0 && pos == name->m_nSize)
		{
			pBackref->m_number = (int)num;
			BufferData_Release(name);
		}
		else
		{
			BufferData_Append(&pData->m_namedbackreflist, &pBackref, 1);
		}

		return (PElxInterface)pBackref;
	}
	else
	{
		unsigned int nbackref = 0;
		int i;

		for(i=0; i<3; i++)
		{
			if(pData->curr.ch >= '0' && pData->curr.ch <= '9')
				nbackref = nbackref * 10 + (pData->curr.ch - '0');
			else
				break;

			BuildContext_MoveNext(pData);
		}

		return BuildContext_Keep(pData, ElxBackRefer_New(nbackref, pData->m_nSizeOfT, *pFlags & 0x10, *pFlags & 0x08));
	}
}

PElxInterface BuildContext_BuildSimple(PBuildContext pData, int * pFlags)
{
	PElxInterface pelx;
	SBufferData fixed;
	BufferData_Init(&fixed, pData->m_nSizeOfT);

	while( ! IsCurrChar(pData, 0, 1) )
	{
		if(pData->curr.type == 0)
		{
			if(IsNextChar(pData, '{', 1) || IsNextChar(pData, '?', 1) || IsNextChar(pData, '*', 1) || IsNextChar(pData, '+', 1))
			{
				if(fixed.m_nSize != 0)
					break;
			}

			BufferData_Append(&fixed, (*pData->m_ltochar)(&pData->curr.ch), 1);
			BuildContext_MoveNext(pData);
		}
		else
		{
			if(pData->curr.ch == ')' || pData->curr.ch == '|' || fixed.m_nSize > 0)
				break;

			switch(pData->curr.ch)
			{
			case '(':
				return BuildContext_BuildRecursive(pData, pFlags);

			case '[':
			case '.':
			case 'w':
			case 'W':
			case 's':
			case 'S':
			case 'd':
			case 'D':
				return BuildContext_BuildCharset(pData, pFlags);

			case '^':
			case '$':
			case 'A':
			case 'Z':
			case 'z':
			case 'b':
			case 'B':
			case 'G':
				return BuildContext_BuildBoundary(pData, pFlags);

			case '\\':
			case 'k':
			case 'g':
				return BuildContext_BuildBackref(pData, pFlags);

			default:
				BufferData_Append(&fixed, (*pData->m_ltochar)(&pData->curr.ch), 1);
				BuildContext_MoveNext(pData);
				break;
			}
		}
	}

	if(fixed.m_nSize > 0)
		pelx = BuildContext_Keep(pData, ElxString_New(fixed.m_pT, fixed.m_nSize, pData->m_nSizeOfT, *pFlags & 0x10, *pFlags & 0x08));
	else
		pelx = BuildContext_Keep(pData, ElxEmpty_New());

	BufferData_Release(&fixed);
	return pelx;
}

PElxInterface BuildContext_BuildRepeat(PBuildContext pData, int * pFlags)
{
	PElxInterface pSimple = BuildContext_BuildSimple(pData, pFlags);
	long nMin, nMax;

	if(pData->curr.type == 0)
		return pSimple;

	switch(pData->curr.ch)
	{
	case '{':
		{
			SStringBufData buf;
			int red, pos;

			StringBufData_Init(&buf, pData->m_nSizeOfT);

			BuildContext_MoveNext(pData);
			while( ! IsCurrChar(pData, 0, 1) && ! IsCurrChar(pData, '}', 1) )
			{
				BufferData_Append(&buf, (*pData->m_ltochar)(&pData->curr.ch), 1);
				BuildContext_MoveNext(pData);
			}
			BuildContext_MoveNext(pData);

			pos = BuildContext_ReadDec(pData, buf.m_pT, buf.m_nSize, &nMin);
			if(pos == 0)
				red = 0;
			else if((pData->m_getch)(&buf, pos) != ',')
				red = 1;
			else
			{
				pos ++;

				if( ! BuildContext_ReadDec(pData, BufferData_GetAt(&buf, pos), buf.m_nSize - pos, &nMax) )
					red = 2;
				else
					red = 3;
			}

			if(red  <=  1 ) nMax = nMin;
			if(red  ==  2 ) nMax = INT_MAX;
			if(nMax < nMin) nMax = nMin;

			BufferData_Release(&buf);
		}
		break;

	case '?':
		nMin = 0;
		nMax = 1;
		BuildContext_MoveNext(pData);
		break;

	case '*':
		nMin = 0;
		nMax = INT_MAX;
		BuildContext_MoveNext(pData);
		break;

	case '+':
		nMin = 1;
		nMax = INT_MAX;
		BuildContext_MoveNext(pData);
		break;

	default:
		return pSimple;
	}

	if(nMax == 0)
		return BuildContext_Keep(pData, ElxEmpty_New());

	nMin = min(nMin, INT_MAX);
	nMax = min(nMax, INT_MAX);

	if(nMin == nMax)
	{
		if(IsCurrChar(pData, '?', 1) || IsCurrChar(pData, '+', 1))
			BuildContext_MoveNext(pData);

		return BuildContext_Keep(pData, ElxRepeat_New(pSimple, nMin));
	}

	if(IsCurrChar(pData, '?', 1))
	{
		BuildContext_MoveNext(pData);
		return BuildContext_Keep(pData, ElxReluctant_New(pSimple, nMin, nMax));
	}
	else if(IsCurrChar(pData, '+', 1))
	{
		BuildContext_MoveNext(pData);
		return BuildContext_Keep(pData, ElxPossessive_New(pSimple, nMin, nMax));
	}
	else
	{
		return BuildContext_Keep(pData, ElxGreedy_New(pSimple, nMin, nMax));
	}
}

PElxInterface BuildContext_BuildList(PBuildContext pData, int * pFlags)
{
	PElxInterface pListOne;

	if(IsCurrChar(pData, 0, 1) || IsCurrChar(pData, '|', 1) || IsCurrChar(pData, ')', 1))
		return BuildContext_Keep(pData, ElxEmpty_New());

	pListOne = BuildContext_BuildRepeat(pData, pFlags);

	if( ! IsCurrChar(pData, 0, 1) && ! IsCurrChar(pData, '|', 1) && ! IsCurrChar(pData, ')', 1) )
	{
		PElxList pList = (PElxList)BuildContext_Keep(pData, ElxList_New(*pFlags & 0x10));
		BufferData_Append(&pList->m_elxlist, &pListOne, 1);

		while( ! IsCurrChar(pData, 0, 1) && ! IsCurrChar(pData, '|', 1) && ! IsCurrChar(pData, ')', 1) )
		{
			pListOne = BuildContext_BuildRepeat(pData, pFlags);
			BufferData_Append(&pList->m_elxlist, &pListOne, 1);
		}

		return (PElxInterface)pList;
	}

	return pListOne;
}

PElxInterface BuildContext_BuildAlternative(PBuildContext pData, int nFlags)
{
	PElxInterface pAlternativeOne;

	if(IsCurrChar(pData, 0, 1))
		return BuildContext_Keep(pData, ElxEmpty_New());

	pAlternativeOne = BuildContext_BuildList(pData, &nFlags);

	if(IsCurrChar(pData, '|', 1))
	{
		PElxAlternative pAlternative = (PElxAlternative)BuildContext_Keep(pData, ElxAlternative_New());
		BufferData_Append(&pAlternative->m_elxlist, &pAlternativeOne, 1);

		while(IsCurrChar(pData, '|', 1))
		{
			BuildContext_MoveNext(pData);

			pAlternativeOne = BuildContext_BuildList(pData, &nFlags);
			BufferData_Append(&pAlternative->m_elxlist, &pAlternativeOne, 1);
		}

		return (PElxInterface)pAlternative;
	}

	return pAlternativeOne;
}

long BuildContext_GetChar1(void * pBuffer, int at)
{
	return *(char*)BufferRefData_GetAt((PBufferRefData)pBuffer, at, 1);
}

long BuildContext_GetChar2(void * pBuffer, int at)
{
	return *(short*)BufferRefData_GetAt((PBufferRefData)pBuffer, at, 2);
}

long BuildContext_GetChar4(void * pBuffer, int at)
{
	return *(long*)BufferRefData_GetAt((PBufferRefData)pBuffer, at, 4);
}

void * BuildContext_LongToCharBig1(long * l)
{
	return ((char*)l) + 3;
}

void * BuildContext_LongToCharBig2(long * l)
{
	return ((char*)l) + 2;
}

void * BuildContext_LongToCharBig4(long * l)
{
	return l;
}

void * BuildContext_LongToChar(long * l)
{
	return l;
}

void BuildContext_Build(PBuildContext pData, void * pStr, int nLen, int nFlags)
{
	int i;
	BuildContext_Reset(pData);

	pData->m_pattern.m_pT    = pStr;
	pData->m_pattern.m_nSize = nLen;
	pData->m_nflags          = nFlags;

	for(i=0; i<3; i++) BuildContext_MoveNext(pData);

	pData->m_ptopelx = BuildContext_BuildAlternative(pData, nFlags);

	BufferData_Prepare(&pData->m_grouplist, 0, 0);
	*(PElxInterface*)BufferData_GetAt(&pData->m_grouplist, 0) = pData->m_ptopelx;

	/* append named to unnamed */
	pData->m_ngroupcount = pData->m_grouplist.m_nSize;
	BufferData_Prepare(&pData->m_grouplist, pData->m_nmaxnumber + pData->m_namedlist.m_nSize, 0);

	for(i=0; i<pData->m_namedlist.m_nSize; i++)
	{
		int find_same_name;

		PElxList    plist  = (PElxList   )BufferData_GetPointerAt(&pData->m_namedlist, i);
		PElxBracket pleft  = (PElxBracket)BufferData_GetPointerAt(&plist->m_elxlist, 0);
		PElxBracket pright = (PElxBracket)BufferData_GetPointerAt(&plist->m_elxlist, 2);

		*(PElxInterface*)BufferData_GetAt(&pData->m_grouplist, pData->m_ngroupcount++) = (PElxInterface)plist;

		if(pleft->m_number > 0)
			continue;

		find_same_name = BuildContext_GetNamedNumber(pData, &pleft->m_name);
		if(find_same_name >= 0)
		{
			pleft ->m_number = pright->m_number = find_same_name;
		}
		else
		{
			pData->m_nmaxnumber ++;
			pleft ->m_number = pright->m_number = pData->m_nmaxnumber;
		}
	}

	/* get max group number */
	for(i=1; i<pData->m_ngroupcount; i++)
	{
		PElxList    plist  = (PElxList   )BufferData_GetPointerAt(&pData->m_grouplist, i);
		PElxBracket pleft  = (PElxBracket)BufferData_GetPointerAt(&plist->m_elxlist, 0);

		if(pleft->m_number > pData->m_nmaxnumber)
		{
			pData->m_nmaxnumber = pleft->m_number;
		}
	}

	/* connect recursive */
	for(i=0; i<pData->m_recursivelist.m_nSize; i++)
	{
		PElxRecursive pRecursive = (PElxRecursive)BufferData_GetPointerAt(&pData->m_recursivelist, i);

		if(pRecursive->m_number == -3)
			pRecursive->m_number = BuildContext_GetNamedNumber(pData, &pRecursive->m_name);

		if(pRecursive->m_number == 0)
			pRecursive->m_pelx = pData->m_ptopelx;
		else
		{
			int j;
			for(j=1; j<pData->m_grouplist.m_nSize; j++)
			{
				PElxList    plist  = (PElxList   )BufferData_GetPointerAt(&pData->m_grouplist, i);
				PElxBracket pleft  = (PElxBracket)BufferData_GetPointerAt(&plist->m_elxlist, 0);

				if(pRecursive->m_number == pleft->m_number)
				{
					pRecursive->m_pelx = (PElxInterface)plist;
					break;
				}
			}
		}
	}

	/* named backref */
	for(i=0; i<pData->m_namedbackreflist.m_nSize; i++)
	{
		PElxBackRefer pBackref = (PElxBackRefer)BufferData_GetPointerAt(&pData->m_namedbackreflist, i);
		pBackref->m_number = BuildContext_GetNamedNumber(pData, &pBackref->m_name);
	}

	/* named condition */
	for(i=0; i<pData->m_namedconditionlist.m_nSize; i++)
	{
		PElxConditional pConditional = (PElxConditional)BufferData_GetPointerAt(&pData->m_namedconditionlist, i);
		int nn = BuildContext_GetNamedNumber(pData, &pConditional->m_name);

		if(nn >= 0)
		{
			pConditional->m_number  = nn;
			pConditional->m_pelxask = 0;
		}
	}
}

void BuildContext_Init(PBuildContext pData, int nSizeOfT)
{
	memset(pData, 0, sizeof(SBuildContext));

	pData->m_nSizeOfT = nSizeOfT;

	BufferData_Init(&pData->m_objlist           , sizeof(PElxInterface));
	BufferData_Init(&pData->m_grouplist         , sizeof(PElxInterface));;
	BufferData_Init(&pData->m_recursivelist     , sizeof(PElxInterface));
	BufferData_Init(&pData->m_namedlist         , sizeof(PElxInterface));
	BufferData_Init(&pData->m_namedbackreflist  , sizeof(PElxInterface));
	BufferData_Init(&pData->m_namedconditionlist, sizeof(PElxInterface));

	switch(nSizeOfT)
	{
	case 1:
		pData->m_compare = StringRefData_Compare1I;
		pData->m_getch   = BuildContext_GetChar1;
		pData->m_ltochar = little_endian() ? BuildContext_LongToChar : BuildContext_LongToCharBig1;
		break;

	case 2:
		pData->m_compare = StringRefData_Compare2I;
		pData->m_getch   = BuildContext_GetChar2;
		pData->m_ltochar = little_endian() ? BuildContext_LongToChar : BuildContext_LongToCharBig2;
		break;

	case 4:
		pData->m_compare = StringRefData_Compare4I;
		pData->m_getch   = BuildContext_GetChar4;
		pData->m_ltochar = little_endian() ? BuildContext_LongToChar : BuildContext_LongToCharBig4;
		break;
	}
}

PBuildContext BuildContext_New(int nSizeOfT)
{
	PBuildContext pData = (PBuildContext)malloc(sizeof(SBuildContext));
	BuildContext_Init(pData, nSizeOfT);
	return pData;
}

typedef PBufferData   result_handle;
typedef PBuildContext regexp_handle;
typedef PMatchContext context_handle;

#define max(a,b)    (((a) > (b)) ? (a) : (b))
#define min(a,b)    (((a) < (b)) ? (a) : (b))

#ifdef __cplusplus
extern "C" {
#endif

regexp_handle regexp_create(int size_of_t)
{
	return BuildContext_New(size_of_t);
}

void regexp_compile(regexp_handle regexp, void * szregexp, int nlength, int nflags)
{
	BuildContext_Build(regexp, szregexp, nlength, nflags);
}

context_handle regexp_prepare(regexp_handle regexp, void * szstring, int nlength, int nstart, context_handle context)
{
	if(context == 0)
		context = MatchContext_New();

	context->m_nParenZindex   = 0;
	context->m_nLastBeginPos  = -1;
	context->m_string.m_pT    = szstring;
	context->m_string.m_nSize = nlength;

	if(nstart < 0)
	{
		if(regexp->m_nflags & 0x10)
		{
			context->m_nBeginPos   = nlength;
			context->m_nCurrentPos = nlength;
		}
		else
		{
			context->m_nBeginPos   = 0;
			context->m_nCurrentPos = 0;
		}
	}
	else
	{
		context->m_nBeginPos   = nstart;
		context->m_nCurrentPos = nstart;
	}

	return context;
}

void regexp_build_result(regexp_handle regexp, context_handle context, result_handle result)
{
	int n;

	BufferData_SetSize(result, 0);
	BufferData_Prepare(result, 2 + (regexp->m_nmaxnumber + 1) * 2 - 1, -1);

	*(int*)BufferData_GetAt(result, 0) = 1;
	*(int*)BufferData_GetAt(result, 1) = regexp->m_nmaxnumber;

	for(n = 0; n <= regexp->m_nmaxnumber; n++)
	{
		PCaptureStackData pdata = MatchContext_GetCaptureData(context, n);
		if(pdata != 0)
		{
			*(int*)BufferData_GetAt(result, 2 + n*2    ) = min(pdata->m_startpos, pdata->m_endpos);
			*(int*)BufferData_GetAt(result, 2 + n*2 + 1) = max(pdata->m_startpos, pdata->m_endpos);
		}
	}
}

result_handle regexp_match_context(regexp_handle regexp, context_handle context, result_handle result)
{
	int endpos, delta;

	if(result == 0)
		result = BufferData_New(0, 0, sizeof(int));

	BufferData_SetSize(result, 0);

	if(regexp->m_nflags & 0x10)
	{
		endpos = -1;
		delta  = -1;
	}
	else
	{
		endpos = context->m_string.m_nSize + 1;
		delta  = 1;
	}

	while(context->m_nCurrentPos != endpos)
	{
		SCaptureStackData cdata;

		BufferData_SetSize(&context->m_captureindex, 0);
		BufferData_SetSize(&context->m_stack       , 0);
		BufferData_SetSize(&context->m_capturestack, 0);

		cdata.m_number   = 0;
		cdata.m_startpos = context->m_nCurrentPos;
		cdata.m_endpos   = -1;
		cdata.m_zindex   = -1;
		BufferData_Append(&context->m_capturestack, &cdata, sizeof(SCaptureStackData));

		BufferData_Prepare(&context->m_captureindex, regexp->m_nmaxnumber, -1);
		*(int*)BufferData_GetAt(&context->m_captureindex, 0) = 0;

		if(ElxInterface_Match(regexp->m_ptopelx, context))
		{
			/* zero width */
			if(context->m_nLastBeginPos == context->m_nBeginPos && context->m_nBeginPos == context->m_nCurrentPos)
			{
				context->m_nCurrentPos += delta;
				continue;
			}

			/* pos */
			context->m_nLastBeginPos = context->m_nBeginPos;
			context->m_nBeginPos     = context->m_nCurrentPos;

			/* group 0 enclose */
			((PCaptureStackData)BufferData_GetAt(&context->m_capturestack, 0))->m_endpos = context->m_nCurrentPos;

			/* match result */
			regexp_build_result(regexp, context, result);

			/* return */
			break;
		}
		else
		{
			context->m_nCurrentPos += delta;
		}
	}

	return result;
}

void regexp_release(regexp_handle regexp)
{
	BuildContext_Delete(regexp);
}

void regexp_release_result(result_handle result)
{
	BufferData_Delete(result);
}

void regexp_release_context(context_handle context)
{
	MatchContext_Delete(context);
}

result_handle regexp_match(regexp_handle regexp, void * szstring, int nlength, int nstart, result_handle result)
{
	context_handle context = regexp_prepare(regexp, szstring, nlength, nstart, 0);
	result = regexp_match_context(regexp, context, result);
	regexp_release_context(context);

	return result;
}

result_handle regexp_match_exact(regexp_handle regexp, void * szstring, int nlength, result_handle result)
{
	int endpos;

	context_handle context = regexp_prepare(regexp, szstring, nlength, -1, 0);

	if(result == 0)
		result = BufferData_New(0, 0, sizeof(int));

	BufferData_SetSize(result, 0);

	if(regexp->m_nflags & 0x10)
	{
		endpos = -1;
	}
	else
	{
		endpos = context->m_string.m_nSize + 1;
	}

	/* do match */
	{
		SCaptureStackData cdata;

		BufferData_SetSize(&context->m_captureindex, 0);
		BufferData_SetSize(&context->m_stack       , 0);
		BufferData_SetSize(&context->m_capturestack, 0);

		cdata.m_number   = 0;
		cdata.m_startpos = context->m_nCurrentPos;
		cdata.m_endpos   = -1;
		cdata.m_zindex   = -1;
		BufferData_Append(&context->m_capturestack, &cdata, sizeof(SCaptureStackData));

		BufferData_Prepare(&context->m_captureindex, regexp->m_nmaxnumber, -1);
		*(int*)BufferData_GetAt(&context->m_captureindex, 0) = 0;

		if(ElxInterface_Match(regexp->m_ptopelx, context))
		{
			while(context->m_nCurrentPos != endpos)
			{
				if( ! ElxInterface_MatchNext(regexp->m_ptopelx, context) )
					return result;

				/* zero width */
				if(context->m_nLastBeginPos == context->m_nBeginPos && context->m_nBeginPos == context->m_nCurrentPos)
					return result;

				/* pos */
				context->m_nLastBeginPos = context->m_nCurrentPos;
			}

			/* group 0 enclose */
			((PCaptureStackData)BufferData_GetAt(&context->m_capturestack, 0))->m_endpos = context->m_nCurrentPos;

			/* match result */
			regexp_build_result(regexp, context, result);
		}
	}

	return result;
}

int result_ismatched(result_handle result)
{
	return result->m_nSize >= 1 ? BufferData_GetIntAt(result, 0) : 0;
}

int result_maxgroupnumber(result_handle result)
{
	return result->m_nSize >= 2 ? BufferData_GetIntAt(result, 1) : 0;
}

int result_getstart(result_handle result)
{
	return result->m_nSize >= 3 ? BufferData_GetIntAt(result, 2) : -1;
}

int result_getend(result_handle result)
{
	return result->m_nSize >= 4 ? BufferData_GetIntAt(result, 3) : -1;
}

int result_getgroupstart(result_handle result, int ngroupnumber)
{
	int p = 2 + ngroupnumber * 2;
	return result->m_nSize > p ? BufferData_GetIntAt(result, p) : -1;
}

int result_getgroupend(result_handle result, int ngroupnumber)
{
	int p = 2 + ngroupnumber * 2 + 1;
	return result->m_nSize > p ? BufferData_GetIntAt(result, p) : -1;
}

#ifdef __cplusplus
}
#endif
